@@ -43,6 +43,7 @@
 #    include <libutil.h>
 #  endif
 #endif
+#include "anjuta-utils-priv.h"
 
 #include <assert.h>
 #include <termios.h>
@@ -63,6 +64,7 @@
 
 #define ANJUTA_PIXMAP_PASSWORD "password.png"
 #define FILE_BUFFER_SIZE 1024
+#define FILE_INPUT_BUFFER_SIZE		1048576
 #ifndef __MAX_BAUD
 #ifdef __CYGWIN__
 #define __MAX_BAUD B256000
@@ -115,7 +117,7 @@ struct _AnjutaLauncherPriv
 	gboolean child_has_terminated;
 	
 	/* Synchronization in progress */
-	gboolean in_synchronization;
+	gboolean in_cleanup;
 	guint completion_check_timeout;
 
 	/* Terminate child on child exit */
@@ -187,7 +189,7 @@ anjuta_launcher_initialize (AnjutaLauncher *obj)
 	obj->priv->child_has_terminated = TRUE;
 	
 	/* Synchronization in progress */
-	obj->priv->in_synchronization = FALSE;
+	obj->priv->in_cleanup = FALSE;
 	obj->priv->completion_check_timeout = -1;
 	
 	/* Terminate child on child exit */
@@ -362,6 +364,28 @@ anjuta_launcher_send_stdin (AnjutaLauncher *launcher, const gchar * input_str)
 	anjuta_launcher_send_ptyin (launcher, input_str);
 }
 
+/**
+ * anjuta_launcher_send_stdin:
+ * @launcher: a #AnjutaLancher object.
+ * 
+ * Sends a EOF to Standard input of the process currently being executed.
+ */
+
+void 
+anjuta_launcher_send_stdin_eof (AnjutaLauncher *launcher)
+{
+	GError* err = NULL;
+	g_io_channel_shutdown (launcher->priv->pty_channel, TRUE,
+						   &err);
+	g_io_channel_unref (launcher->priv->pty_channel);
+	launcher->priv->pty_channel = NULL;
+	
+	if (err)
+	{
+		g_warning ("g_io_channel_shutdown () failed: %s", err->message);
+	}
+}
+
 /**
  * anjuta_launcher_send_ptyin:
  * @launcher: a #AnjutaLancher object.
@@ -445,12 +469,12 @@ anjuta_launcher_get_child_pid (AnjutaLauncher *launcher)
 static void
 anjuta_launcher_synchronize (AnjutaLauncher *launcher)
 {
+	if (launcher->priv->in_cleanup) return;
+
 	if (launcher->priv->child_has_terminated &&
 		launcher->priv->stdout_is_done &&
 		launcher->priv->stderr_is_done)
 	{
-		launcher->priv->in_synchronization = TRUE;
-		
 		if (launcher->priv->completion_check_timeout >= 0)
 			g_source_remove (launcher->priv->completion_check_timeout);
 		launcher->priv->completion_check_timeout = 
@@ -861,18 +885,19 @@ static void
 anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 										gboolean emit_signal)
 {
-	static gboolean block = FALSE;
 	gint child_status, child_pid;
 	time_t start_time;
-	
+
+	if (launcher->priv->in_cleanup)
+		return;
+
+	launcher->priv->in_cleanup = TRUE;
+
+	/* Remove pending timeout */
 	if (launcher->priv->completion_check_timeout >= 0)
 		g_source_remove (launcher->priv->completion_check_timeout);
 	
-	if (block)
-		return;
-	
 	/* Make sure all pending I/O are flushed out */
-	block = TRUE;
 	while (g_main_context_pending (NULL))
 		g_main_context_iteration (NULL, FALSE);
 	
@@ -927,7 +952,7 @@ anjuta_launcher_execution_done_cleanup (AnjutaLauncher *launcher,
 							   child_status,
 							   time (NULL) - start_time);
 		
-	block = FALSE;
+	launcher->priv->in_cleanup = FALSE;
 }
 
 /* Using this function is necessary because
@@ -1088,6 +1113,9 @@ anjuta_launcher_fork (AnjutaLauncher *launcher, gchar *const args[])
 	launcher->priv->stdout_channel = g_io_channel_unix_new (stdout_pipe[0]);
 	launcher->priv->pty_channel = g_io_channel_unix_new (pty_master_fd);
 
+	g_io_channel_set_buffer_size (launcher->priv->pty_channel, FILE_INPUT_BUFFER_SIZE);
+	
+		
 	g_get_charset (&charset);
 	anjuta_launcher_set_encoding (launcher, charset);
 
@@ -1160,7 +1188,7 @@ anjuta_launcher_execute_v (AnjutaLauncher *launcher, gchar *const argv[],
 	launcher->priv->stdout_is_done = FALSE;
 	launcher->priv->stderr_is_done = FALSE;
 	launcher->priv->child_has_terminated = FALSE;
-	launcher->priv->in_synchronization = FALSE;
+	launcher->priv->in_cleanup = FALSE;
 	launcher->priv->output_callback = callback;
 	launcher->priv->callback_data = callback_data;
 	
