@@ -87,11 +87,13 @@ libanjuta_iface_cclosure_marshal_VOID__BOOLEAN_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM (./libanjuta-iface-marshallers.list:3) */
+/* VOID:BOXED (./libanjuta-iface-marshallers.list:3) */
 
-/* VOID:INT (./libanjuta-iface-marshallers.list:4) */
+/* VOID:ENUM (./libanjuta-iface-marshallers.list:4) */
 
-/* VOID:INT,BOOLEAN,INT,INT,STRING (./libanjuta-iface-marshallers.list:5) */
+/* VOID:INT (./libanjuta-iface-marshallers.list:5) */
+
+/* VOID:INT,BOOLEAN,INT,INT,STRING (./libanjuta-iface-marshallers.list:6) */
 void
 libanjuta_iface_cclosure_marshal_VOID__INT_BOOLEAN_INT_INT_STRING (GClosure     *closure,
                                                                    GValue       *return_value,
@@ -134,7 +136,7 @@ libanjuta_iface_cclosure_marshal_VOID__INT_BOOLEAN_INT_INT_STRING (GClosure
             data2);
 }
 
-/* VOID:INT,CHAR (./libanjuta-iface-marshallers.list:6) */
+/* VOID:INT,CHAR (./libanjuta-iface-marshallers.list:7) */
 void
 libanjuta_iface_cclosure_marshal_VOID__INT_CHAR (GClosure     *closure,
                                                  GValue       *return_value,
@@ -171,11 +173,54 @@ libanjuta_iface_cclosure_marshal_VOID__INT_CHAR (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER (./libanjuta-iface-marshallers.list:7) */
+/* VOID:INT,INT,UINT,STRING,UINT (./libanjuta-iface-marshallers.list:8) */
+void
+libanjuta_iface_cclosure_marshal_VOID__INT_INT_UINT_STRING_UINT (GClosure     *closure,
+                                                                 GValue       *return_value,
+                                                                 guint         n_param_values,
+                                                                 const GValue *param_values,
+                                                                 gpointer      invocation_hint,
+                                                                 gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_INT_UINT_STRING_UINT) (gpointer     data1,
+                                                               gint         arg_1,
+                                                               gint         arg_2,
+                                                               guint        arg_3,
+                                                               gpointer     arg_4,
+                                                               guint        arg_5,
+                                                               gpointer     data2);
+  register GMarshalFunc_VOID__INT_INT_UINT_STRING_UINT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
 
-/* VOID:STRING (./libanjuta-iface-marshallers.list:8) */
+  g_return_if_fail (n_param_values == 6);
 
-/* VOID:STRING,STRING (./libanjuta-iface-marshallers.list:9) */
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_INT_UINT_STRING_UINT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_uint (param_values + 3),
+            g_marshal_value_peek_string (param_values + 4),
+            g_marshal_value_peek_uint (param_values + 5),
+            data2);
+}
+
+/* VOID:POINTER (./libanjuta-iface-marshallers.list:9) */
+
+/* VOID:STRING (./libanjuta-iface-marshallers.list:10) */
+
+/* VOID:STRING,STRING (./libanjuta-iface-marshallers.list:11) */
 void
 libanjuta_iface_cclosure_marshal_VOID__STRING_STRING (GClosure     *closure,
                                                       GValue       *return_value,
@@ -212,61 +257,20 @@ libanjuta_iface_cclosure_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,UINT,UINT (./libanjuta-iface-marshallers.list:10) */
+/* VOID:UINT,INT (./libanjuta-iface-marshallers.list:12) */
 void
-libanjuta_iface_cclosure_marshal_VOID__STRING_UINT_UINT (GClosure     *closure,
-                                                         GValue       *return_value,
-                                                         guint         n_param_values,
-                                                         const GValue *param_values,
-                                                         gpointer      invocation_hint,
-                                                         gpointer      marshal_data)
+libanjuta_iface_cclosure_marshal_VOID__UINT_INT (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_UINT_UINT) (gpointer     data1,
-                                                       gpointer     arg_1,
-                                                       guint        arg_2,
-                                                       guint        arg_3,
-                                                       gpointer     data2);
-  register GMarshalFunc_VOID__STRING_UINT_UINT callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-
-  g_return_if_fail (n_param_values == 4);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_VOID__STRING_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
-
-  callback (data1,
-            g_marshal_value_peek_string (param_values + 1),
-            g_marshal_value_peek_uint (param_values + 2),
-            g_marshal_value_peek_uint (param_values + 3),
-            data2);
-}
-
-/* VOID:UINT (./libanjuta-iface-marshallers.list:11) */
-
-/* VOID:UINT,UINT (./libanjuta-iface-marshallers.list:12) */
-void
-libanjuta_iface_cclosure_marshal_VOID__UINT_UINT (GClosure     *closure,
-                                                  GValue       *return_value,
-                                                  guint         n_param_values,
-                                                  const GValue *param_values,
-                                                  gpointer      invocation_hint,
-                                                  gpointer      marshal_data)
-{
-  typedef void (*GMarshalFunc_VOID__UINT_UINT) (gpointer     data1,
-                                                guint        arg_1,
-                                                guint        arg_2,
-                                                gpointer     data2);
-  register GMarshalFunc_VOID__UINT_UINT callback;
+  typedef void (*GMarshalFunc_VOID__UINT_INT) (gpointer     data1,
+                                               guint        arg_1,
+                                               gint         arg_2,
+                                               gpointer     data2);
+  register GMarshalFunc_VOID__UINT_INT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -282,13 +286,52 @@ libanjuta_iface_cclosure_marshal_VOID__UINT_UINT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__UINT_INT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_marshal_value_peek_uint (param_values + 1),
-            g_marshal_value_peek_uint (param_values + 2),
+            g_marshal_value_peek_int (param_values + 2),
             data2);
 }
 
-/* VOID:VOID (./libanjuta-iface-marshallers.list:13) */
+/* VOID:UINT,STRING,UINT (./libanjuta-iface-marshallers.list:13) */
+void
+libanjuta_iface_cclosure_marshal_VOID__UINT_STRING_UINT (GClosure     *closure,
+                                                         GValue       *return_value,
+                                                         guint         n_param_values,
+                                                         const GValue *param_values,
+                                                         gpointer      invocation_hint,
+                                                         gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT_STRING_UINT) (gpointer     data1,
+                                                       guint        arg_1,
+                                                       gpointer     arg_2,
+                                                       guint        arg_3,
+                                                       gpointer     data2);
+  register GMarshalFunc_VOID__UINT_STRING_UINT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT_STRING_UINT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_string (param_values + 2),
+            g_marshal_value_peek_uint (param_values + 3),
+            data2);
+}
+
+/* VOID:VOID (./libanjuta-iface-marshallers.list:14) */
 
