@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     debug_tree.c
     Copyright (C) 2006  Sébastien Granjoux
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 // TODO:
@@ -34,7 +35,7 @@
 #include "plugin.h"
 #include "memory.h"
 
-#define DEBUG
+/*#define DEBUG*/
 #include <libanjuta/anjuta-debug.h>
 
 #include <libgnome/gnome-i18n.h>
@@ -77,10 +78,9 @@ struct _CommonDebugTree {
 
 /* The debug tree object */
 struct _DebugTree {
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	AnjutaPlugin *plugin;
 	GtkWidget* view;        /* the tree widget */
-	GSList* free_list;
 	gboolean auto_expand;
 };
 
@@ -332,7 +332,7 @@ delete_parent(GtkTreeModel *model, GtkTreePath* path,
 			{
 				/* Object has been created in debugger and is not a child
 			 	* (destroyed with their parent) */
-				ianjuta_variable_debugger_delete_var (IANJUTA_VARIABLE_DEBUGGER (tree->debugger), data->name, NULL);
+				dma_queue_delete_variable (tree->debugger, data->name);
 			}
 		}				
 
@@ -483,9 +483,7 @@ destroy_non_analyzed (DebugTree *tree, GtkTreeModel* model)
 		gtk_tree_model_get(model, &iter, DTREE_ENTRY_COLUMN, &data, -1);
 		if ((data != NULL) && (data->analyzed == FALSE))
 		{
-			debug_tree_remove (tree, &iter);
-			
-			success = gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+			success = debug_tree_remove (tree, &iter);
 		}
 		else
 		{
@@ -565,7 +563,7 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 		 * created at this time, so remove it now */
 		if ((pack->tree->debugger) && (variable->name))
 		{
-			ianjuta_variable_debugger_delete_var (IANJUTA_VARIABLE_DEBUGGER (pack->tree->debugger), variable->name, NULL);
+			dma_queue_delete_variable (pack->tree->debugger, variable->name);
 		}
 		dma_variable_packet_free (pack);
 		
@@ -592,12 +590,11 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 		DmaVariablePacket *pack_child =
                   dma_variable_packet_new(pack->model, &iter, pack->tree, data);
 
-		ianjuta_variable_debugger_list_children (
-				IANJUTA_VARIABLE_DEBUGGER (pack_child->tree->debugger),
+		dma_queue_list_children (
+				pack_child->tree->debugger,
 				variable->name,
 				(IAnjutaDebuggerCallback)gdb_var_list_children,
-				pack_child,
-				NULL);
+				pack_child);
 	}
 	else if (variable->children > 0)
 	{
@@ -613,12 +610,11 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 	 * reusing same packet if possible */
 	if (variable->value == NULL)
 	{
-		ianjuta_variable_debugger_evaluate (
-				IANJUTA_VARIABLE_DEBUGGER (pack->tree->debugger),
+		dma_queue_evaluate_variable (
+				pack->tree->debugger,
 				variable->name,
 				(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-				pack,
-				NULL);
+				pack);
 	}
 	else
 	{
@@ -645,12 +641,11 @@ on_treeview_row_expanded       (GtkTreeView     *treeview,
 		DmaVariablePacket *pack;
 					
 		pack = dma_variable_packet_new(model, iter, tree, data);
-		ianjuta_variable_debugger_list_children (
-						IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_list_children (
+						tree->debugger,
 						data->name,
 						(IAnjutaDebuggerCallback)gdb_var_list_children,
-						pack,
-						NULL);
+						pack);
 	}
 	else
 	{
@@ -706,14 +701,13 @@ on_debug_tree_value_changed (GtkCellRendererText *cell,
 		if (item != NULL)
 		{
 			/* Variable is valid */
-			ianjuta_variable_debugger_assign (IANJUTA_VARIABLE_DEBUGGER (tree->debugger), item->name, text, NULL);
+			dma_queue_assign_variable (tree->debugger, item->name, text);
 			tran = dma_variable_packet_new(model, &iter, tree, item);
-			ianjuta_variable_debugger_evaluate (
-					IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+			dma_queue_evaluate_variable (
+					tree->debugger,
 					item->name,
 					(IAnjutaDebuggerCallback) gdb_var_evaluate_expression,
-					tran,
-					NULL);
+					tran);
 		}
 	}
 }
@@ -791,8 +785,7 @@ debug_tree_remove_all (DebugTree *tree)
 	g_return_if_fail (tree->view);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
-	my_gtk_tree_model_foreach_child (model, NULL, delete_parent, tree);
-	gtk_tree_store_clear (GTK_TREE_STORE (model));
+	debug_tree_remove_model (tree, model);
 }
 
 static gboolean
@@ -971,12 +964,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 				DmaVariablePacket *pack;
 		
 				pack = dma_variable_packet_new(model, &iter, tree, data);
-				ianjuta_variable_debugger_create (
-						IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+				dma_queue_create_variable (
+						tree->debugger,
 						var->expression,
 						(IAnjutaDebuggerCallback)gdb_var_create,
-						pack,
-						NULL);
+						pack);
 			}
 			else
 			{
@@ -987,12 +979,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 					DmaVariablePacket *pack =
 					
 					pack = dma_variable_packet_new(model, &iter, tree, data);
-					ianjuta_variable_debugger_evaluate (
-							IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+					dma_queue_evaluate_variable (
+							tree->debugger,
 							var->name,
 							(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-							pack,
-							NULL);
+							pack);
 				}
 				if (var->children == -1)
 				{
@@ -1000,12 +991,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 					DmaVariablePacket *pack =
 					
 					pack = dma_variable_packet_new(model, &iter, tree, data);
-					ianjuta_variable_debugger_list_children (
-							IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+					dma_queue_list_children (
+							tree->debugger,
 							var->name,
 							(IAnjutaDebuggerCallback)gdb_var_list_children,
-							pack,
-							NULL);
+							pack);
 				}
 			}
 		}
@@ -1059,7 +1049,7 @@ static void
 on_debug_tree_changed (gpointer data, gpointer user_data)
 {
 	IAnjutaDebuggerVariable *var = (IAnjutaDebuggerVariable *)data;
-	
+
 	if (var->name != NULL)
 	{
 		/* Search corresponding variable in one tree */
@@ -1070,7 +1060,7 @@ on_debug_tree_changed (gpointer data, gpointer user_data)
 			GtkTreeIter iter;
 			GtkTreeModel *model;
 			
-			model = gtk_tree_view_get_model (GTK_TREE_VIEW (((DebugTree *)tree->data)->view));
+			model =  GTK_TREE_MODEL (tree->data);
 			
 			if (debug_tree_find_name (model, &iter, var->name))
 			{
@@ -1141,11 +1131,10 @@ debug_tree_update_all (DebugTree* tree)
 	if (tree->debugger != NULL)
 	{
 		/* Update if debugger is connected */
-		ianjuta_variable_debugger_update (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_update_variable (
+				tree->debugger,
 				(IAnjutaDebuggerCallback)on_debug_tree_update_all,
-				tree,
-				NULL);
+				tree);
 	}
 }
 
@@ -1197,19 +1186,20 @@ debug_tree_get_current (DebugTree *tree, GtkTreeIter* iter)
 	return get_current_iter (GTK_TREE_VIEW (tree->view), iter);
 }
 
-void
+/* Return TRUE if iter is still valid (point to next item) */
+gboolean
 debug_tree_remove (DebugTree *tree, GtkTreeIter* iter)
 {
 	GtkTreeModel *model;
 
-	g_return_if_fail (tree);
-	g_return_if_fail (tree->view);
-	g_return_if_fail (iter);
+	g_return_val_if_fail (tree, FALSE);
+	g_return_val_if_fail (tree->view, FALSE);
+	g_return_val_if_fail (iter, FALSE);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
 	
 	delete_parent (model, NULL, iter, tree);
-	gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
+	return gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
 }
 
 gboolean
@@ -1233,12 +1223,11 @@ debug_tree_update (DebugTree* tree, GtkTreeIter* iter, gboolean force)
 		gtk_tree_model_get (model, iter, VARIABLE_COLUMN, &exp, -1);
 		pack = dma_variable_packet_new(model, iter, tree, data);
 		data->modified = TRUE;
-		ianjuta_variable_debugger_create (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_create_variable (
+				tree->debugger,
 				exp,
 				(IAnjutaDebuggerCallback)gdb_var_create,
-				pack,
-				NULL);
+				pack);
 		g_free (exp);
 		
 		return FALSE;
@@ -1248,12 +1237,11 @@ debug_tree_update (DebugTree* tree, GtkTreeIter* iter, gboolean force)
 		DmaVariablePacket *pack = dma_variable_packet_new(model, iter, tree, data);
 		refresh = data->modified != (data->changed != FALSE);
 		data->modified = (data->changed != FALSE);
-		ianjuta_variable_debugger_evaluate (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_evaluate_variable (
+				tree->debugger,
 				data->name,
 				(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-				pack,
-				NULL);
+				pack);
 	}
 	else
 	{
@@ -1318,7 +1306,7 @@ debug_tree_get_auto_update (DebugTree* this, GtkTreeIter* iter)
 }
 
 void
-debug_tree_connect (DebugTree *this, IAnjutaDebugger* debugger)
+debug_tree_connect (DebugTree *this, DmaDebuggerQueue* debugger)
 {
 	this->debugger = debugger;
 }
@@ -1329,6 +1317,78 @@ debug_tree_disconnect (DebugTree *this)
 	this->debugger = NULL;
 }
 
+GtkTreeModel *
+debug_tree_get_model (DebugTree *tree)
+{
+	return gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+}
+
+void
+debug_tree_set_model (DebugTree *tree, GtkTreeModel *model)
+{
+	gtk_tree_view_set_model (GTK_TREE_VIEW (tree->view), model);
+}
+
+void
+debug_tree_new_model (DebugTree *tree)
+{
+	GtkTreeModel * model = GTK_TREE_MODEL (gtk_tree_store_new
+		                     (N_COLUMNS, 
+	                          G_TYPE_STRING, 
+	                          G_TYPE_STRING,
+                              G_TYPE_STRING,
+				              G_TYPE_BOOLEAN,
+			                  G_TYPE_POINTER));
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW (tree->view), model);
+}
+
+void
+debug_tree_remove_model (DebugTree *tree, GtkTreeModel *model)
+{
+	my_gtk_tree_model_foreach_child (model, NULL, delete_parent, tree);
+	gtk_tree_store_clear (GTK_TREE_STORE (model));
+}
+
+gchar *
+debug_tree_get_selected (DebugTree *tree)
+{
+	GtkTreeIter iter;
+	gchar *exp = NULL;
+
+	if (get_current_iter (GTK_TREE_VIEW (tree->view), &iter))
+	{
+		GtkTreeModel *model;
+
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+		if (model != NULL)
+		{
+			gtk_tree_model_get(model, &iter, VARIABLE_COLUMN, &exp, -1);
+		}
+	}
+	
+	return exp;
+}
+
+gchar *
+debug_tree_get_first (DebugTree *tree)
+{
+	GtkTreeIter iter;
+	gchar *exp = NULL;
+	GtkTreeModel *model;
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+	if (model != NULL)
+	{
+		if (gtk_tree_model_get_iter_first (model, &iter))
+		{
+			gtk_tree_model_get(model, &iter, VARIABLE_COLUMN, &exp, -1);
+		}
+	}
+	
+	return exp;
+}
+
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
@@ -1337,13 +1397,15 @@ DebugTree *
 debug_tree_new_with_view (AnjutaPlugin *plugin, GtkTreeView *view)
 {
 	DebugTree *tree = g_new0 (DebugTree, 1);
+	GtkTreeModel *model;	
 
 	tree->plugin = plugin;
 	tree->view = debug_tree_create(tree, view);
 	tree->auto_expand = FALSE;
 
-	/* Add this tree in list */
-	gTreeList = g_list_prepend (gTreeList, tree);
+	/* Add this model in list */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+	gTreeList = g_list_prepend (gTreeList, model);
 	
 	/* Connect signal */
     g_signal_connect(GTK_TREE_VIEW (tree->view), "row_expanded", G_CALLBACK (on_treeview_row_expanded), tree);
@@ -1362,22 +1424,24 @@ debug_tree_new (AnjutaPlugin* plugin)
 
 /* DebugTree destructor */
 void
-debug_tree_free (DebugTree * d_tree)
+debug_tree_free (DebugTree * tree)
 {
+	GtkTreeModel *model;
 	// AnjutaUI *ui;
 	
-	g_return_if_fail (d_tree);
+	g_return_if_fail (tree);
 
-	debug_tree_remove_all (d_tree);
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+	debug_tree_remove_all (tree);
 
 	/* Remove from list */
-	gTreeList = g_list_remove (gTreeList, d_tree);
+	gTreeList = g_list_remove (gTreeList, model);
 	
-	g_signal_handlers_disconnect_by_func (GTK_TREE_VIEW (d_tree->view),
-				  G_CALLBACK (on_treeview_row_expanded), d_tree);
+	g_signal_handlers_disconnect_by_func (GTK_TREE_VIEW (tree->view),
+				  G_CALLBACK (on_treeview_row_expanded), tree);
 	
-	gtk_widget_destroy (d_tree->view);
+	gtk_widget_destroy (tree->view);
 	
 	
-	g_free (d_tree);
+	g_free (tree);
 }
