@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
  
 #ifdef HAVE_CONFIG_H
@@ -39,6 +39,7 @@
 #include "sparse_view.h"
 
 #include "plugin.h"
+#include "queue.h"
 
 /* Constants
  *---------------------------------------------------------------------------*/
@@ -69,7 +70,7 @@ typedef struct _DmaDisassemblyViewClass DmaDisassemblyViewClass;
 
 struct _DmaDisassemble
 {
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	DebugManagerPlugin *plugin;
 	GtkWidget *window;
 	GtkWidget *menu;
@@ -83,7 +84,7 @@ struct _DmaDisassemble
 struct _DmaDisassemblyBuffer
 {
 	DmaSparseBuffer parent;
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	gboolean pending;
 };
 
@@ -150,7 +151,6 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	gint line = -1;
 	DmaDisassemblyBufferNode *node = (DmaDisassemblyBufferNode *)iter->node;
 	
-	DEBUG_PRINT("iter_refresh iter->node %p (%x, %d), base %x offset %d", iter->node, iter->node != NULL ? iter->node->lower : 0, iter->node != NULL ? ((DmaDisassemblyBufferNode *)iter->node)->size : 0, iter->base, iter->offset);
 	if (iter->node != NULL)
 	{
 		/* Find line corresponding to base */
@@ -219,7 +219,6 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	}
 
 	/* Try to reduce offset */
-	DEBUG_PRINT("iter_refresh offset iter->node %p, base %x offset %d line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
 	if (line != -1)
 	{
 		if (iter->offset > 0)
@@ -329,7 +328,6 @@ dma_disassembly_iter_refresh (DmaSparseIter *iter)
 	}
 	
 	/* Round offset */
-	DEBUG_PRINT("iter_refresh round iter->node %p, base %x offset %d line %d line %d", iter->node, iter->base, iter->offset, iter->line, line);
 	if (iter->offset < 0)
 	{
 		gulong address;
@@ -410,6 +408,7 @@ on_disassemble (const IAnjutaDebuggerDisassembly *block, DmaSparseBufferTranspor
 	}
 	
 	/* Find following block */
+	DEBUG_PRINT("trans %p buffer %p trans->buffer %p trans->start %ul", trans, buffer, trans == NULL ? NULL : trans->buffer, trans == NULL ? 0 : trans->start);
 	next = dma_sparse_buffer_lookup (DMA_SPARSE_BUFFER (buffer), trans->start + trans->length - 1);
 	if ((next != NULL) && (next->upper <= trans->start)) next = NULL;
 	
@@ -418,8 +417,6 @@ on_disassemble (const IAnjutaDebuggerDisassembly *block, DmaSparseBufferTranspor
 		gulong address = trans->start;
 		gint len;
 			
-		DEBUG_PRINT ("Create a dummy disassembly node");
-			
 		/* Create a dummy node */
 		len = (trans->length + DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH - 1) / DMA_DISASSEMBLY_DEFAULT_LINE_LENGTH;
 		node = (DmaDisassemblyBufferNode *)g_malloc0 (sizeof(DmaDisassemblyBufferNode) + sizeof(DmaDisassemblyLine) * len);
@@ -448,7 +445,6 @@ on_disassemble (const IAnjutaDebuggerDisassembly *block, DmaSparseBufferTranspor
 		guint size = 0;
 		guint line = 0;
 		
-		DEBUG_PRINT ("on disassemble size %d", block->size);
 		/* Compute size of all data */
 		/* use size -1 because last block has no data (NULL) */
 		for (i = trans->tag == DMA_DISASSEMBLY_KEEP_ALL ? 0 : 4; i < block->size - 1; i++)
@@ -533,8 +529,6 @@ dma_disassembly_buffer_insert_line (DmaSparseIter *iter, GtkTextIter *dst)
 				gulong end_adr;
 				gint margin;
 			
-				DEBUG_PRINT("no data at address %lx + %ld", iter->base, iter->offset);
-				
 				/* If following line is define, get a block stopping here */
 				dma_sparse_iter_copy (&end, iter);
 				margin = 0;
@@ -568,8 +562,8 @@ dma_disassembly_buffer_insert_line (DmaSparseIter *iter, GtkTextIter *dst)
 				{
 					trans->length++;
 				}
-				DEBUG_PRINT("get disassemble %lx %lx %ld", start_adr, end_adr, trans->length);
-				ianjuta_cpu_debugger_disassemble ((IAnjutaCpuDebugger *)dis->debugger, start_adr, end_adr + 1 - start_adr, (IAnjutaDebuggerCallback)on_disassemble, trans, NULL);
+				DEBUG_PRINT("get disassemble %lx %lx %ld trans %p buffer %p", start_adr, end_adr, trans->length, trans, trans->buffer);
+				dma_queue_disassemble (dis->debugger, start_adr, end_adr + 1 - start_adr, (IAnjutaDebuggerCallback)on_disassemble, trans);
 			}
 		}
 		else
@@ -634,7 +628,7 @@ dma_disassembly_buffer_get_type (void)
 
 
 static DmaDisassemblyBuffer*
-dma_disassembly_buffer_new (IAnjutaDebugger *debugger, gulong lower, gulong upper)
+dma_disassembly_buffer_new (DmaDebuggerQueue *debugger, gulong lower, gulong upper)
 {
 	DmaDisassemblyBuffer *buffer;
 
@@ -649,20 +643,13 @@ dma_disassembly_buffer_new (IAnjutaDebugger *debugger, gulong lower, gulong uppe
 	return buffer;
 }
 	
-static void
-dma_disassembly_buffer_free (DmaDisassemblyBuffer *buffer)
-{
-	g_object_unref (buffer);
-}
-
-
 /* Disassembly view object
  *---------------------------------------------------------------------------*/
 
 struct _DmaDisassemblyView
 {
 	DmaSparseView parent;
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	gboolean pending;
 };
 
@@ -724,7 +711,7 @@ dma_disassembly_view_get_type (void)
 
 
 static DmaDisassemblyView*
-dma_disassembly_view_new_with_buffer (IAnjutaDebugger *debugger, DmaSparseBuffer *buffer)
+dma_disassembly_view_new_with_buffer (DmaDebuggerQueue *debugger, DmaSparseBuffer *buffer)
 {
 	DmaDisassemblyView *view;
 
@@ -738,12 +725,6 @@ dma_disassembly_view_new_with_buffer (IAnjutaDebugger *debugger, DmaSparseBuffer
 	return view;
 }
 	
-static void
-dma_disassembly_view_free (DmaDisassemblyView *view)
-{
-	g_object_unref (view);
-}
-
 /* Private functions
  *---------------------------------------------------------------------------*/
 
@@ -758,62 +739,17 @@ on_breakpoint_changed (DmaDisassemble *self, IAnjutaDebuggerBreakpoint *bp)
 {
 	g_return_if_fail (bp != NULL);
 	
-	dma_disassemble_unmark (self, bp->address, IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
-	dma_disassemble_unmark (self, bp->address, IANJUTA_MARKABLE_BREAKPOINT_ENABLED);
-	if (bp->type != IANJUTA_DEBUGGER_BREAK_REMOVED)
+	dma_sparse_view_unmark (self->view, bp->address, IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
+	dma_sparse_view_unmark (self->view, bp->address, IANJUTA_MARKABLE_BREAKPOINT_ENABLED);
+	if (!(bp->type & IANJUTA_DEBUGGER_BREAK_REMOVED))
 	{
-		dma_disassemble_mark (self, bp->address, bp->enable ? IANJUTA_MARKABLE_BREAKPOINT_ENABLED : IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
+		dma_sparse_view_mark (self->view, bp->address, bp->enable ? IANJUTA_MARKABLE_BREAKPOINT_ENABLED : IANJUTA_MARKABLE_BREAKPOINT_DISABLED);
 	}
 }
 
-static GtkWidget*
-create_disassemble_gui (DmaDisassemble *self)
-{
-	GtkWidget *dataview;
-
-	dataview = GTK_WIDGET (dma_disassembly_view_new_with_buffer (self->debugger, self->buffer));
-
-	/* Add register window */
-	self->window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->window),
-									GTK_POLICY_AUTOMATIC,
-									GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self->window),
-									 GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER (self->window), GTK_WIDGET (dataview));
-	
-	anjuta_shell_add_widget (ANJUTA_PLUGIN (self->plugin)->shell,
-							 self->window,
-                             "AnjutaDebuggerDisassemble", _("Disassembly"),
-                             NULL, ANJUTA_SHELL_PLACEMENT_LEFT,
-							 NULL);
-
-
-	DMA_DISASSEMBLY_VIEW (dataview)->pending = FALSE;
-	
-	return dataview;
-}
-
-static void
-on_debugger_started (DmaDisassemble *self)
-{
-	self->buffer = DMA_SPARSE_BUFFER (dma_disassembly_buffer_new (self->debugger, 0x00000000U,0xFFFFFFFFU));
-
-	self->view = DMA_SPARSE_VIEW (create_disassemble_gui (self));
-
-	g_signal_connect_swapped (self->plugin, "breakpoint-changed", G_CALLBACK (on_breakpoint_changed), self);
-	
-	g_signal_connect (G_OBJECT (self->buffer), 
-			"changed", 
-			G_CALLBACK (on_disassembly_buffer_changed), 
-			self->view);
-}
-
 static void
 destroy_disassemble_gui (DmaDisassemble *self)
 {
-	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_breakpoint_changed), self);
-
 	/* Destroy menu */
 	if (self->menu != NULL)
 	{
@@ -827,78 +763,120 @@ destroy_disassemble_gui (DmaDisassemble *self)
 		self->window = NULL;
 		self->view = NULL;
 	}
+	
+	/* Destroy buffer */
+	if (self->buffer)
+	{
+		dma_sparse_buffer_free (DMA_SPARSE_BUFFER (self->buffer));
+		self->buffer = NULL;
+	}
+}
+
+static void
+on_program_running (DmaDisassemble *self)
+{
+	dma_sparse_view_delete_all_markers (self->view, IANJUTA_MARKABLE_PROGRAM_COUNTER);	
+}
+
+static void
+on_program_moved (DmaDisassemble *self, guint pid, guint tid, guint address, const gchar* file, guint line)
+{
+	dma_sparse_view_delete_all_markers (self->view, IANJUTA_MARKABLE_PROGRAM_COUNTER);	
+	
+	if (address != 0)
+	{
+		dma_sparse_view_mark (self->view, address, IANJUTA_MARKABLE_PROGRAM_COUNTER);
+		dma_sparse_view_goto (self->view, address);
+	}
+}
+
+static void
+on_location_changed (DmaDisassemble *self, guint address, const gchar* uri, guint line)
+{
+	dma_sparse_view_goto (self->view, address);
 }
 
 static void
 on_debugger_stopped (DmaDisassemble *self)
 {
-	g_signal_handlers_disconnect_by_func (self->buffer,
-			on_disassembly_buffer_changed,
-			self->view);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_debugger_stopped), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_breakpoint_changed), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_program_moved), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_location_changed), self);
 
-	dma_sparse_buffer_free (DMA_SPARSE_BUFFER (self->buffer));
-	self->buffer = NULL;
+	dma_sparse_view_delete_all_markers (self->view, IANJUTA_MARKABLE_PROGRAM_COUNTER);	
 	
 	destroy_disassemble_gui (self);
 }
 
+
+static gboolean
+create_disassemble_gui (DmaDisassemble *self)
+{
+	GtkWidget *dataview;
+
+	g_return_val_if_fail (self->buffer == NULL, FALSE);
+	g_return_val_if_fail (self->window == NULL, FALSE);
+	
+	/* Create buffer */
+	self->buffer = DMA_SPARSE_BUFFER (dma_disassembly_buffer_new (self->debugger, 0x00000000U,0xFFFFFFFFU));
+	if (self->buffer == NULL) return FALSE;		
+	
+	dataview = GTK_WIDGET (dma_disassembly_view_new_with_buffer (self->debugger, self->buffer));
+	self->view = DMA_SPARSE_VIEW (dataview);
+	DMA_DISASSEMBLY_VIEW (dataview)->pending = FALSE;
+	g_signal_connect (G_OBJECT (self->buffer), "changed", G_CALLBACK (on_disassembly_buffer_changed), self->view);
+
+	/* Add disassembly window */
+	self->window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->window),
+									GTK_POLICY_AUTOMATIC,
+									GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self->window),
+									 GTK_SHADOW_IN);
+	gtk_container_add (GTK_CONTAINER (self->window), GTK_WIDGET (dataview));
+	
+	anjuta_shell_add_widget (ANJUTA_PLUGIN (self->plugin)->shell,
+							 self->window,
+                             "AnjutaDebuggerDisassemble", _("Disassembly"),
+                             NULL, ANJUTA_SHELL_PLACEMENT_LEFT,
+							 NULL);
+	
+	return TRUE;
+}
+
+static void
+on_debugger_started (DmaDisassemble *self)
+{
+	if (!(dma_debugger_queue_get_feature (self->debugger) & HAS_CPU)) return;
+
+	if (!create_disassemble_gui (self)) return;
+
+	/* Connect signals */
+	g_signal_connect_swapped (self->plugin, "debugger-stopped", G_CALLBACK (on_debugger_stopped), self);
+	g_signal_connect_swapped (self->plugin, "breakpoint-changed", G_CALLBACK (on_breakpoint_changed), self);
+	g_signal_connect_swapped (self->plugin, "program-running", G_CALLBACK (on_program_running), self);
+	g_signal_connect_swapped (self->plugin, "program-moved", G_CALLBACK (on_program_moved), self);
+	g_signal_connect_swapped (self->plugin, "location-changed", G_CALLBACK (on_location_changed), self);
+}
+
 /* Public functions
  *---------------------------------------------------------------------------*/
 
-void
-dma_disassemble_mark (DmaDisassemble *self, guint address, gint marker)
-{
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (self->view != NULL);
-	
-	dma_sparse_view_mark (self->view, address, marker);
-}
-
-void
-dma_disassemble_unmark (DmaDisassemble *self, guint address, gint marker)
-{
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (self->view != NULL);
-	
-	dma_sparse_view_unmark (self->view, address, marker);
-}
-
-void
-dma_disassemble_clear_all_mark (DmaDisassemble *self, gint marker)
-{
-	g_return_if_fail (self != NULL);
-	/* Accept to clear mark without a view, used at initialization */
-	if (self->view != NULL)
-		dma_sparse_view_delete_all_markers (self->view, marker);
-}
-
-void
-dma_disassemble_goto_address (DmaDisassemble *self, guint address)
-{
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (self->view != NULL);
-	
-	dma_sparse_view_goto (self->view, address);
-}
-
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
 DmaDisassemble*
-dma_disassemble_new(AnjutaPlugin *plugin, IAnjutaDebugger *debugger)
+dma_disassemble_new(DebugManagerPlugin *plugin)
 {
 	DmaDisassemble* self;
 	
 	self = g_new0 (DmaDisassemble, 1);
 	
-	self->debugger = debugger;
-	if (debugger != NULL) g_object_ref (debugger);
-	self->plugin = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
-	self->buffer = NULL;
-	self->view = NULL;
+	self->plugin = plugin;
+	self->debugger = dma_debug_manager_get_queue (plugin);;
 
-	g_signal_connect_swapped (self->debugger, "debugger-started", G_CALLBACK (on_debugger_started), self);
-	g_signal_connect_swapped (self->debugger, "debugger-stopped", G_CALLBACK (on_debugger_stopped), self);
+	g_signal_connect_swapped (self->plugin, "debugger-started", G_CALLBACK (on_debugger_started), self);
 	
 	return self;
 }
@@ -908,10 +886,10 @@ dma_disassemble_free(DmaDisassemble* self)
 {
 	g_return_if_fail (self != NULL);
 
+	g_signal_handlers_disconnect_matched (self->plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);	
+	
 	destroy_disassemble_gui (self);
 
-	if (self->debugger != NULL)	g_object_unref (self->debugger);
-	
 	g_free(self);
 }
 
