@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -37,7 +37,7 @@
 #include "registers.h"
 #include "utilities.h"
 #include "start.h"
-#include "debugger.h"
+#include "queue.h"
 
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
@@ -69,7 +69,6 @@ struct _DebugManagerPlugin
 
 	/* Debugger queue */
 	DmaDebuggerQueue *queue;
-	IAnjutaDebugger *debugger;
 	
 	/* Menu item */
 	gint uiid;
@@ -116,19 +115,9 @@ struct _DebugManagerPluginClass
 /* Private functions
  *---------------------------------------------------------------------------*/
 
-#define REGISTER_ICON(icon, stock_id) \
-        pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"icon, NULL); \
-        icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-        gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-        g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-        AnjutaUI *ui;
-        GtkIconFactory *icon_factory;
-        GtkIconSet *icon_set;
-        GdkPixbuf *pixbuf;
         static gboolean registered = FALSE;
 
         if (registered)
@@ -136,18 +125,22 @@ register_stock_icons (AnjutaPlugin *plugin)
         registered = TRUE;
 
         /* Register stock icons */
-        ui = anjuta_shell_get_ui (plugin->shell, NULL);
-        icon_factory = anjuta_ui_get_icon_factory (ui);
+		BEGIN_REGISTER_ICON (plugin)
 		REGISTER_ICON (ICON_FILE, "debugger-icon");
         REGISTER_ICON ("stack.png", "gdb-stack-icon");
         REGISTER_ICON ("locals.png", "gdb-locals-icon");
         REGISTER_ICON ("watch.png", "gdb-watch-icon");
-        REGISTER_ICON ("breakpoint.png", "gdb-breakpoint-toggle");
-		REGISTER_ICON ("detach.png", "debugger-detach");
-		REGISTER_ICON ("step-into.png", "debugger-step-into");
-		REGISTER_ICON ("step-out.png", "debugger-step-out");
-		REGISTER_ICON ("step-over.png", "debugger-step-over");
-		REGISTER_ICON ("run-to-cursor.png", "debugger-run-to-cursor");
+        REGISTER_ICON_FULL ("anjuta-breakpoint-toggle", "gdb-breakpoint-toggle");
+        REGISTER_ICON_FULL ("anjuta-breakpoint-clear", "gdb-breakpoint-clear");		
+    	/* We have no -24 version for the next two */
+		REGISTER_ICON ("anjuta-breakpoint-disabled-16.png", "gdb-breakpoint-disabled");
+        REGISTER_ICON ("anjuta-breakpoint-enabled-16.png", "gdb-breakpoint-enabled");				
+		REGISTER_ICON_FULL ("anjuta-attach", "debugger-attach");
+		REGISTER_ICON_FULL ("anjuta-step-into", "debugger-step-into");
+		REGISTER_ICON_FULL ("anjuta-step-out", "debugger-step-out");
+		REGISTER_ICON_FULL ("anjuta-step-over", "debugger-step-over");
+		REGISTER_ICON_FULL ("anjuta-run-to-cursor", "debugger-run-to-cursor");
+		END_REGISTER_ICON
 }
 
 /* Program counter functions
@@ -175,13 +168,6 @@ show_program_counter_in_editor(DebugManagerPlugin *self)
 	}
 }
 
-static void
-show_program_counter_in_disassembler(DebugManagerPlugin *self)
-{
-	dma_disassemble_mark (self->disassemble, self->pc_address, IANJUTA_MARKABLE_PROGRAM_COUNTER);
-	dma_disassemble_goto_address (self->disassemble, self->pc_address);
-}
-
 static void
 hide_program_counter_in_editor(DebugManagerPlugin *self)
 {
@@ -200,12 +186,6 @@ hide_program_counter_in_editor(DebugManagerPlugin *self)
 	}
 }
 
-static void
-hide_program_counter_in_disassembler(DebugManagerPlugin *self)
-{
-	dma_disassemble_clear_all_mark (self->disassemble, IANJUTA_MARKABLE_PROGRAM_COUNTER);
-}
-
 static void
 set_program_counter(DebugManagerPlugin *self, const gchar* file, guint line, guint address)
 {
@@ -214,17 +194,13 @@ set_program_counter(DebugManagerPlugin *self, const gchar* file, guint line, gui
 
 	/* Remove previous marker */
 	hide_program_counter_in_editor (self);
-	hide_program_counter_in_disassembler (self);
 	if (self->pc_editor != NULL)
 	{
-		g_object_remove_weak_pointer (G_OBJECT (self->pc_editor), (gpointer *)&self->pc_editor);
+		g_object_remove_weak_pointer (G_OBJECT (self->pc_editor), (gpointer *)(gpointer)&self->pc_editor);
 		self->pc_editor = NULL;
 	}
 	self->pc_address = address;
 
-	if (address != 0)
-		show_program_counter_in_disassembler (self);
-
 	if (file != NULL)
 	{
 		docman = anjuta_shell_get_interface (ANJUTA_PLUGIN (self)->shell, IAnjutaDocumentManager, NULL);
@@ -238,7 +214,7 @@ set_program_counter(DebugManagerPlugin *self, const gchar* file, guint line, gui
 			if (editor != NULL)
 			{
 				self->pc_editor = editor;
-				g_object_add_weak_pointer (G_OBJECT (editor), (gpointer)&self->pc_editor);
+				g_object_add_weak_pointer (G_OBJECT (editor), (gpointer)(gpointer)&self->pc_editor);
 				self->pc_line = line;
 				show_program_counter_in_editor (self);
 			}
@@ -284,16 +260,23 @@ static void
 value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 							const GValue *value, gpointer data)
 {
-	IAnjutaEditor *editor;
+	GObject *editor;
 	DebugManagerPlugin *self;
 
-	editor = IANJUTA_EDITOR (g_value_get_object (value));
 	self = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
-	self->current_editor = editor;
-	g_object_add_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)&self->current_editor);
+
+	editor = g_value_get_object (value);
+	if (!IANJUTA_IS_EDITOR(editor))
+	{
+		self->current_editor = NULL;
+		return;
+	}
+							 
+	self->current_editor = IANJUTA_EDITOR (editor);
+	g_object_add_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)(gpointer)&self->current_editor);
 		
     /* Restore breakpoints */
-	breakpoints_dbase_set_all_in_editor (self->breakpoints, editor);
+	breakpoints_dbase_set_all_in_editor (self->breakpoints, self->current_editor);
 	
 	/* Restore program counter marker */
 	show_program_counter_in_editor (self);
@@ -312,9 +295,9 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	
 		hide_program_counter_in_editor (self);
 
-		g_object_remove_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)&self->current_editor);
-		self->current_editor = NULL;
+		g_object_remove_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)(gpointer)&self->current_editor);
 	}
+	self->current_editor = NULL;
 }
 
 static void
@@ -330,9 +313,9 @@ enable_log_view (DebugManagerPlugin *this, gboolean enable)
 			this->view = ianjuta_message_manager_add_view (man, _("Debugger Log"), ICON_FILE, NULL);
 			if (this->view != NULL)
 			{
-				//g_signal_connect (G_OBJECT (this->view), "buffer_flushed", G_CALLBACK (on_message_buffer_flushed), this);
-				g_object_add_weak_pointer (G_OBJECT (this->view), (gpointer *)&this->view);
-				ianjuta_debugger_enable_log (this->debugger, this->view, NULL);
+				/*g_signal_connect (G_OBJECT (this->view), "buffer_flushed", G_CALLBACK (on_message_buffer_flushed), this);*/
+				g_object_add_weak_pointer (G_OBJECT (this->view), (gpointer *)(gpointer)&this->view);
+				dma_queue_enable_log (this->queue, this->view);
 			}
 		}
 		else
@@ -350,7 +333,10 @@ enable_log_view (DebugManagerPlugin *this, gboolean enable)
 			ianjuta_message_manager_remove_view (man, this->view, NULL);
 			this->view = NULL;
 		}
-		ianjuta_debugger_disable_log (this->debugger, NULL);
+		if (this->queue != NULL)
+		{
+			dma_queue_disable_log (this->queue);
+		}
 	}
 }
 
@@ -364,9 +350,9 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 		return;
 
 	/* Close debugger when session changed */
-	if (plugin->debugger)
+	if (plugin->queue)
 	{
-		ianjuta_debugger_abort (plugin->debugger, NULL);
+		dma_queue_abort (plugin->queue);
 	}
 }
 
@@ -374,27 +360,6 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 /* State functions
  *---------------------------------------------------------------------------*/
 
-static void
-dma_plugin_debugger_ready (DebugManagerPlugin *self, IAnjutaDebuggerStatus stat)
-{
-	AnjutaStatus* status;
-
-	if ((stat == IANJUTA_DEBUGGER_BUSY) != self->busy)
-	{
-		status = anjuta_shell_get_status(ANJUTA_PLUGIN (self)->shell, NULL);
-		if (!self->busy)
-		{
-			anjuta_status_busy_push (status);
-			self->busy = TRUE;
-		}
-		else
-		{
-			anjuta_status_busy_pop (status);
-			self->busy = FALSE;
-		}
-	}
-}
-
 /* Called when the debugger is started but no program is loaded */
 
 static void
@@ -420,8 +385,6 @@ dma_plugin_debugger_started (DebugManagerPlugin *this)
 	
 	status = anjuta_shell_get_status(ANJUTA_PLUGIN (this)->shell, NULL);
 	anjuta_status_set_default (status, _("Debugger"), _("Started"));
-	
-	dma_plugin_debugger_ready (this, IANJUTA_DEBUGGER_STARTED);
 }
 
 /* Called when a program is loaded */
@@ -436,20 +399,13 @@ dma_plugin_program_loaded (DebugManagerPlugin *this)
 	
 	if (this->sharedlibs == NULL)
 	{
-		this->sharedlibs = sharedlibs_new (this->debugger);
+		this->sharedlibs = sharedlibs_new (this);
 	}
 	if (this->signals == NULL)
 	{
-		this->signals = signals_new (this->debugger);
+		this->signals = signals_new (this);
 	}
 
-	/* Connect components */
-	breakpoints_dbase_connect (this->breakpoints, this->debugger);
-	expr_watch_connect (this->watch, this->debugger);
-//	locals_connect (this->locals, this->debugger);
-//	stack_trace_connect (this->stack, this->debugger);
-//	cpu_registers_connect (this->registers, this->debugger);
-						
 	/* Update ui */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
 	gtk_action_group_set_sensitive (this->loaded_group, TRUE);
@@ -492,11 +448,6 @@ dma_plugin_program_stopped (DebugManagerPlugin *this)
 	
 	DEBUG_PRINT ("DMA: dma_plugin_program_broken");
 	
-	expr_watch_cmd_queqe (this->watch);
-//	locals_update (this->locals);
-//	stack_trace_update (this->stack);
-//	cpu_registers_update (this->registers);
-
 	/* Update ui */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
 	gtk_action_group_set_sensitive (this->loaded_group, TRUE);
@@ -510,15 +461,9 @@ dma_plugin_program_stopped (DebugManagerPlugin *this)
 /* Called when the program postion change */
 
 static void
-dma_plugin_location_changed (DebugManagerPlugin *this, const gchar* file, guint line, guint address)
+dma_plugin_program_moved (DebugManagerPlugin *this, guint pid, guint tid, guint address, const gchar* file, guint line)
 {
-	gchar *msg;
-
-	DEBUG_PRINT ("DMA: dma_plugin_location_changed %s %d %x", file, line, address);
-	
-	msg = g_strdup_printf (_("Location: %s, line %d\n"), file, line);
-	dma_debugger_message (this->queue, msg);
-	g_free (msg);
+	DEBUG_PRINT ("DMA: dma_plugin_program_moved %s %d %x", file, line, address);
 
 	set_program_counter (this, file, line, address);
 }
@@ -564,7 +509,7 @@ dma_plugin_program_unload (DebugManagerPlugin *this)
 /* Called when the debugger is stopped */
 
 static void
-dma_plugin_debugger_stopped (DebugManagerPlugin *self, gint status)
+dma_plugin_debugger_stopped (DebugManagerPlugin *self, GError *err)
 {
 	AnjutaUI *ui;
 	GtkAction *action;
@@ -595,13 +540,11 @@ dma_plugin_debugger_stopped (DebugManagerPlugin *self, gint status)
 	state = anjuta_shell_get_status(ANJUTA_PLUGIN (self)->shell, NULL);
 	anjuta_status_set_default (state, _("Debugger"), NULL);
 	
-	dma_plugin_debugger_ready (self, IANJUTA_DEBUGGER_STOPPED);
-	
 	/* Display a warning if debugger stop unexpectedly */
-	if (status != 0)
+	if (err != NULL)
 	{
 		GtkWindow *parent = GTK_WINDOW (ANJUTA_PLUGIN(self)->shell);
-		anjuta_util_dialog_error (parent, _("gdb terminated unexpectedly with status %d\n"), status);
+		anjuta_util_dialog_error (parent, _("Debugger terminated with error %d: %s\n"), err->code, err->message);
 	}
 		
 }
@@ -618,6 +561,24 @@ dma_plugin_signal_received (DebugManagerPlugin *self, const gchar *name, const g
 	}
 }
 
+/* Called when the user want to go to another location */
+
+static void
+dma_plugin_location_changed (DebugManagerPlugin *self, guint address, const gchar* uri, guint line)
+{
+	/* Go to location in editor */
+	if (uri != NULL)
+	{
+		IAnjutaDocumentManager *docman;
+
+        docman = anjuta_shell_get_interface (ANJUTA_PLUGIN(self)->shell, IAnjutaDocumentManager, NULL);
+        if (docman)
+        {
+			ianjuta_document_manager_goto_file_line (docman, uri, line, NULL);
+        }
+	}
+}
+
 /* Start/Stop menu functions
  *---------------------------------------------------------------------------*/
 	
@@ -651,10 +612,9 @@ on_attach_to_project_action_activate (GtkAction* action, DebugManagerPlugin* thi
 static void
 on_debugger_stop_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
+	if (plugin->start)
 	{
-		ianjuta_debugger_interrupt (plugin->debugger, NULL);
-		ianjuta_debugger_quit (plugin->debugger, NULL);
+		dma_quit_debugger (plugin->start);
 	}
 }
 
@@ -664,29 +624,29 @@ on_debugger_stop_activate (GtkAction* action, DebugManagerPlugin* plugin)
 static void
 on_run_continue_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
-		ianjuta_debugger_run (plugin->debugger, NULL /* TODO */);
+	if (plugin->queue)
+		dma_queue_run (plugin->queue);
 }
 
 static void
 on_step_in_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
-		ianjuta_debugger_step_in (plugin->debugger, NULL /* TODO */);
+	if (plugin->queue)
+		dma_queue_step_in (plugin->queue);
 }
 
 static void
 on_step_over_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
-		ianjuta_debugger_step_over (plugin->debugger, NULL /* TODO */);
+	if (plugin->queue)
+		dma_queue_step_over (plugin->queue);
 }
 
 static void
 on_step_out_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
-		ianjuta_debugger_step_out (plugin->debugger, NULL /* TODO */);
+	if (plugin->queue)
+		dma_queue_step_out (plugin->queue);
 }
 
 static void
@@ -694,21 +654,25 @@ on_run_to_cursor_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
 	IAnjutaDocumentManager *docman;
 	IAnjutaEditor *editor;
-	const gchar *uri;
+	IAnjutaDocument *document;
+	gchar *uri;
 	gchar *file;
 	gint line;
 
-	if (plugin->debugger == NULL)
+	if (plugin->queue == NULL)
 		return;
 	
 	docman = IANJUTA_DOCUMENT_MANAGER (anjuta_shell_get_object (ANJUTA_PLUGIN (plugin)->shell, "IAnjutaDocumentManager", NULL));
 	if (docman == NULL)
 		return;
 	
-	editor = ianjuta_document_manager_get_current_editor (docman, NULL);
-	if (editor == NULL)
+	document = ianjuta_document_manager_get_current_document (docman, NULL);
+	if (IANJUTA_IS_DOCUMENT(document))
+	{	
+		editor = IANJUTA_EDITOR(document);
+	}
+	else
 		return;
-	
 	uri = ianjuta_file_get_uri (IANJUTA_FILE (editor), NULL);
 	if (uri == NULL)
 		return;
@@ -716,17 +680,16 @@ on_run_to_cursor_action_activate (GtkAction* action, DebugManagerPlugin* plugin)
 	file = gnome_vfs_get_local_path_from_uri (uri);
 	
 	line = ianjuta_editor_get_lineno (editor, NULL);
-	ianjuta_debugger_run_to (plugin->debugger, file, line,
-									  NULL /* TODO */);
+	dma_queue_run_to (plugin->queue, file, line);
+	g_free (uri);
 	g_free (file);
-	
 }
 
 static void
 on_debugger_interrupt_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
-	if (plugin->debugger)
-		ianjuta_debugger_interrupt (plugin->debugger, NULL);
+	if (plugin->queue)
+		dma_queue_interrupt (plugin->queue);
 }
 
 /* Custom command
@@ -739,7 +702,7 @@ on_debugger_command_entry_activate (GtkEntry *entry, DebugManagerPlugin *plugin)
 
         command = gtk_entry_get_text (GTK_ENTRY (entry));
         if (command && strlen (command))
-                ianjuta_debugger_send_command (plugin->debugger, command, NULL);
+                dma_queue_send_command (plugin->queue, command);
         gtk_entry_set_text (entry, "");
 }
 
@@ -780,37 +743,37 @@ on_debugger_dialog_message (const gpointer data, gpointer user_data, GError* err
 static void
 on_info_targets_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_target (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_target (plugin->queue, on_debugger_dialog_message, plugin);
 }
 
 static void
 on_info_program_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_program (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_program (plugin->queue, on_debugger_dialog_message, plugin);
 }
 
 static void
 on_info_udot_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_udot (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_udot (plugin->queue, on_debugger_dialog_message, plugin);
 }
 
 static void
 on_info_variables_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_variables (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_variables (plugin->queue, on_debugger_dialog_message, plugin);
 }
 
 static void
 on_info_frame_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_frame (plugin->debugger, 0, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_frame (plugin->queue, 0, on_debugger_dialog_message, plugin);
 }
 
 static void
 on_info_args_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
-	ianjuta_debugger_info_args (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
+	dma_queue_info_args (plugin->queue, on_debugger_dialog_message, plugin);
 }
 
 /* Other informations
@@ -839,12 +802,6 @@ on_debugger_signals_activate (GtkAction * action, DebugManagerPlugin *plugin)
 	signals_show (plugin->signals);
 }
 
-static void
-on_debugger_ready_signal (IAnjutaDebugger* dbg, IAnjutaDebuggerStatus stat, DebugManagerPlugin *plugin)
-{
-	dma_plugin_debugger_ready (plugin, stat);
-}
-
 /* Actions table
  *---------------------------------------------------------------------------*/
 
@@ -884,7 +841,7 @@ static GtkActionEntry actions_start[] =
 	},
 	{
 		"ActionDebuggerAttachProcess",
-		"debugger-detach",
+		"debugger-attach",
 		N_("_Attach to Process..."),
 		NULL,
 		N_("Attach to a running program"),
@@ -1102,17 +1059,16 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 
 	/* Load debugger */
 	this->queue = dma_debugger_queue_new (plugin);
-	this->debugger = IANJUTA_DEBUGGER (this->queue);
-	g_signal_connect_swapped (this->debugger, "debugger-started", G_CALLBACK (dma_plugin_debugger_started), this);
-	g_signal_connect_swapped (this->debugger, "debugger-stopped", G_CALLBACK (dma_plugin_debugger_stopped), this);
-	g_signal_connect_swapped (this->debugger, "program-loaded", G_CALLBACK (dma_plugin_program_loaded), this);
-	g_signal_connect_swapped (this->debugger, "program-running", G_CALLBACK (dma_plugin_program_running), this);
-	g_signal_connect_swapped (this->debugger, "program-stopped", G_CALLBACK (dma_plugin_program_stopped), this);
-	g_signal_connect_swapped (this->debugger, "program-exited", G_CALLBACK (dma_plugin_program_loaded), this);
-	g_signal_connect_swapped (this->debugger, "location-changed", G_CALLBACK (dma_plugin_location_changed), this);
-	g_signal_connect_swapped (this->debugger, "signal-received", G_CALLBACK (dma_plugin_signal_received), this);
-	g_signal_connect (this->debugger, "debugger-ready", G_CALLBACK (on_debugger_ready_signal), this);
-
+	g_signal_connect (this, "debugger-started", G_CALLBACK (dma_plugin_debugger_started), this);
+	g_signal_connect (this, "debugger-stopped", G_CALLBACK (dma_plugin_debugger_stopped), this);
+	g_signal_connect (this, "program-loaded", G_CALLBACK (dma_plugin_program_loaded), this);
+	g_signal_connect (this, "program-running", G_CALLBACK (dma_plugin_program_running), this);
+	g_signal_connect (this, "program-stopped", G_CALLBACK (dma_plugin_program_stopped), this);
+	g_signal_connect (this, "program-exited", G_CALLBACK (dma_plugin_program_loaded), this);
+	g_signal_connect (this, "program-moved", G_CALLBACK (dma_plugin_program_moved), this);
+	g_signal_connect (this, "signal-received", G_CALLBACK (dma_plugin_signal_received), this);
+	g_signal_connect (this, "location_changed", G_CALLBACK (dma_plugin_location_changed), this);
+	
 	/* Add all our debug manager actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (plugin)->shell, NULL);
 	this->start_group =
@@ -1143,31 +1099,31 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 
 	
 	/* Watch expression */
-	this->watch = expr_watch_new (ANJUTA_PLUGIN (plugin), this->debugger);
+	this->watch = expr_watch_new (ANJUTA_PLUGIN (this));
 	
 	/* Local window */
-	this->locals = locals_new (ANJUTA_PLUGIN (plugin), this->debugger);
+	this->locals = locals_new (this);
 
 	/* Stack trace */
-	this->stack = stack_trace_new (this->debugger, this);
+	this->stack = stack_trace_new (this);
 
 	/* Thread list */
-	this->thread = dma_threads_new (this->debugger, this);
+	this->thread = dma_threads_new (this);
 	
 	/* Create breakpoints list */
 	this->breakpoints = breakpoints_dbase_new (this);
 
 	/* Register list */
-	this->registers = cpu_registers_new (plugin, this->debugger);
+	this->registers = cpu_registers_new (this);
 
 	/* Memory window */
-	this->memory = dma_memory_new (plugin, this->debugger);
+	this->memory = dma_memory_new (this);
 
 	/* Disassembly window */
-	this->disassemble = dma_disassemble_new (plugin, this->debugger);
+	this->disassemble = dma_disassemble_new (this);
 
 	/* Start debugger part */
-	this->start = dma_start_new (plugin, this->debugger);
+	this->start = dma_start_new (this);
 	
 
 	dma_plugin_debugger_stopped (this, 0);
@@ -1203,14 +1159,10 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 
 	/* Stop debugger */
 	dma_plugin_debugger_stopped (this, 0);
-	
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_debugger_started), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_debugger_stopped), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_program_loaded), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_program_running), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_program_stopped), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_location_changed), this);
-	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (on_debugger_ready_signal), this);
+
+	g_signal_handlers_disconnect_matched (plugin->shell, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, plugin);	
+	g_signal_handlers_disconnect_matched (plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, plugin);	
+
 	dma_debugger_queue_free (this->queue);
 	this->queue = NULL;
 
@@ -1252,7 +1204,7 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 
 	if (this->view != NULL)
 	{
-		g_object_remove_weak_pointer (G_OBJECT (this->view), (gpointer*)&this->view);
+		g_object_remove_weak_pointer (G_OBJECT (this->view), (gpointer*)(gpointer)&this->view);
         this->view = NULL;
 	}
 	
@@ -1262,23 +1214,12 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 /* Public functions
  *---------------------------------------------------------------------------*/
 
-void
-dma_debug_manager_goto_code (DebugManagerPlugin *self, const gchar *uri, guint line, guint address)
+DmaDebuggerQueue* 
+dma_debug_manager_get_queue (DebugManagerPlugin *self)
 {
-	/* Go to address in disassembly view */
-	if (address != 0)
-	{
-		dma_disassemble_goto_address (self->disassemble, address);	
-	}
-
-	/* Go to location in editor */
-	if (uri != NULL)
-	{
-		goto_location_in_editor (ANJUTA_PLUGIN (self), uri, line);
-	}
+	return self->queue;
 }
 
-
 /* GObject functions
  *---------------------------------------------------------------------------*/
 
@@ -1296,7 +1237,7 @@ dma_plugin_instance_init (GObject* obj)
 	plugin->uiid = 0;
 	
 	plugin->project_root_uri = NULL;
-	plugin->debugger = NULL;
+	plugin->queue = NULL;
 	plugin->current_editor = NULL;
 	plugin->pc_editor = NULL;
 	plugin->editor_watch_id = 0;
@@ -1334,11 +1275,11 @@ dma_plugin_finalize (GObject* obj)
 
 	if (self->pc_editor != NULL)
 	{
-		g_object_remove_weak_pointer (G_OBJECT (self->pc_editor), (gpointer *)&self->pc_editor);
+		g_object_remove_weak_pointer (G_OBJECT (self->pc_editor), (gpointer *)(gpointer)&self->pc_editor);
 	}
 	if (self->current_editor != NULL)
 	{
-		g_object_remove_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)&self->current_editor);
+		g_object_remove_weak_pointer (G_OBJECT (self->current_editor), (gpointer *)(gpointer)&self->current_editor);
 	}
 	
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (G_OBJECT (obj)));
