@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -34,6 +34,7 @@
 #include <libanjuta/anjuta-debug.h>
 
 #include "utilities.h"
+#include "queue.h"
 
 /* Constants
  *---------------------------------------------------------------------------*/
@@ -48,18 +49,18 @@
 typedef struct _DmaThreadRegisterList
 {
 	GtkTreeModel *model;
-	guint thread;
+	gint thread;
 	guint last_update;
 } DmaThreadRegisterList;
 
 struct _CpuRegisters
 {
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	AnjutaPlugin *plugin;
 	DmaThreadRegisterList* current;
 	GList *list;
 	GtkTreeView *treeview;
-	GtkWidget *scrolledwindow;
+	GtkWidget *window;
 	guint current_update;
 };
 
@@ -121,12 +122,17 @@ on_cpu_registers_updated (const GList *registers, gpointer user_data, GError *er
 	GtkTreeIter iter;
 	gboolean valid;
 
+	if (error != NULL)
+	{
+		/* Command canceled */
+		
+		return;
+	}
+	
 	/* Get first item in list view */
 	valid = gtk_tree_model_get_iter_first (self->current->model, &iter);
 	list = GTK_LIST_STORE (self->current->model);
 
-	if (registers == NULL) return;
-	
 	self->current->last_update = self->current_update;
 	
 	for(node = registers;node != NULL; node = g_list_next (node))
@@ -196,18 +202,17 @@ on_cpu_registers_updated (const GList *registers, gpointer user_data, GError *er
 static void
 cpu_registers_update (CpuRegisters *self)
 {
-	if ((self->debugger != NULL) && GTK_WIDGET_MAPPED (self->scrolledwindow))
+	if (GTK_WIDGET_MAPPED (self->window))
 	{
-		ianjuta_cpu_debugger_update_register (
-				IANJUTA_CPU_DEBUGGER (self->debugger),
+		dma_queue_update_register (
+				self->debugger,
 				(IAnjutaDebuggerCallback)on_cpu_registers_updated,
-				self,
-				NULL);
+				self);
 	}
 }
 
 static DmaThreadRegisterList*
-dma_thread_create_new_register_list (CpuRegisters *self, guint thread)
+dma_thread_create_new_register_list (CpuRegisters *self, gint thread)
 {
 	DmaThreadRegisterList *regs;
 	GtkListStore *store;
@@ -238,14 +243,29 @@ dma_thread_create_new_register_list (CpuRegisters *self, guint thread)
 		
 	if (self->list == NULL)
 	{
+		GError *err = NULL;
+		
 		self->current = regs;
 		
 		/* List is empty, ask debugger to get all register name */
-		ianjuta_cpu_debugger_list_register (
-				IANJUTA_CPU_DEBUGGER (self->debugger),
+		dma_queue_list_register (
+				self->debugger,
 				(IAnjutaDebuggerCallback)on_cpu_registers_updated,
-				self,
-				NULL);
+				self);
+		
+		if (err != NULL)
+		{
+			if (err->code == IANJUTA_DEBUGGER_NOT_IMPLEMENTED)
+			{
+				g_object_unref (G_OBJECT (regs->model));
+				g_free (regs);
+				self->current = NULL;
+				
+				return NULL;
+			}
+			
+			g_error_free (err);
+		}
 	}
 	else
 	{
@@ -283,17 +303,19 @@ static gboolean
 on_find_register_list (gconstpointer a, gconstpointer b)
 {
 	const DmaThreadRegisterList *regs = (const DmaThreadRegisterList *)a;
-	guint thread = (guint)b;
+	guint thread = (gint)b;
 	
 	return regs->thread != thread;
 }
 
 static void
-dma_thread_set_register_list (CpuRegisters *self, guint thread)
+dma_thread_set_register_list (CpuRegisters *self, gint thread)
 {
 	GList *list;
 	DmaThreadRegisterList *regs;
 
+	if (self->current == NULL) return;	/* No register list */
+
 	if (self->current->thread != thread)
 	{
 		list = g_list_find_custom (self->list, (gconstpointer) thread, on_find_register_list);
@@ -335,30 +357,57 @@ on_cpu_registers_changed (GtkCellRendererText *cell,
 		reg.value = text;
 		reg.name = name;
 		
-		ianjuta_cpu_debugger_write_register (IANJUTA_CPU_DEBUGGER (self->debugger), &reg, NULL);
-		ianjuta_cpu_debugger_update_register (
-				IANJUTA_CPU_DEBUGGER (self->debugger),
+		dma_queue_write_register (self->debugger, &reg);
+		dma_queue_update_register (
+				self->debugger,
 				(IAnjutaDebuggerCallback)on_cpu_registers_updated,
-				self,
-				NULL);
+				self);
 		g_free (name);
 	}
 }
 
 static void
-on_program_stopped (CpuRegisters *self, guint thread)
+on_program_moved (CpuRegisters *self, guint pid, gint thread)
 {
 	self->current_update++;
 	dma_thread_set_register_list (self, thread);
 }
 
+static void
+on_frame_changed (CpuRegisters *self, guint frame, gint thread)
+{
+	dma_thread_set_register_list (self, thread);
+}
+
+static void
+destroy_cpu_registers_gui (CpuRegisters *self)
+{
+	/* Destroy register window */
+	if (self->window != NULL)
+	{
+		gtk_widget_destroy (self->window);
+		self->window = NULL;
+	}
+		
+	/* Destroy register list */
+	dma_thread_clear_all_register_list (self);
+}
+
+static void
+on_debugger_stopped (CpuRegisters *self)
+{
+	/* Disconnect signals */
+	g_signal_handlers_disconnect_by_func (self->plugin, on_debugger_stopped, self);
+	g_signal_handlers_disconnect_by_func (self->plugin, on_program_moved, self);
+	g_signal_handlers_disconnect_by_func (self->plugin, on_frame_changed, self);
+
+	destroy_cpu_registers_gui (self);
+}
+
 static void
 on_map (GtkWidget* widget, CpuRegisters *self)
 {
-	if (ianjuta_debugger_get_status (self->debugger, NULL) == IANJUTA_DEBUGGER_PROGRAM_STOPPED)
-	{
-		cpu_registers_update (self);
-	}
+	cpu_registers_update (self);
 }
 
 static void
@@ -383,14 +432,21 @@ cpu_registers_value_cell_data_func (GtkTreeViewColumn *tree_column,
 }
 
 
-static void
+static gboolean
 create_cpu_registers_gui (CpuRegisters *self)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 
+	g_return_val_if_fail (self->window == NULL, FALSE);
+	
 	/* Create list store */
-	dma_thread_create_new_register_list (self, 0);
+	if (dma_thread_create_new_register_list (self, 0) == NULL)
+	{
+		/* Unable to create register list */
+		
+		return FALSE;
+	}
 	
 	/* Create list view */
 	self->treeview = GTK_TREE_VIEW (gtk_tree_view_new_with_model (self->current->model));
@@ -414,72 +470,56 @@ create_cpu_registers_gui (CpuRegisters *self)
 	gtk_tree_view_append_column (self->treeview, column);
 	
 	/* Add register window */
-	self->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->scrolledwindow),
+	self->window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->window),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self->scrolledwindow),
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self->window),
 									 GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER (self->scrolledwindow), GTK_WIDGET (self->treeview));
-	g_signal_connect(self->scrolledwindow, "map", (GCallback) on_map, self);
+	gtk_container_add (GTK_CONTAINER (self->window), GTK_WIDGET (self->treeview));
+	g_signal_connect(self->window, "map", (GCallback) on_map, self);
 
 	anjuta_shell_add_widget (self->plugin->shell,
-							 self->scrolledwindow,
+							 self->window,
                              "AnjutaDebuggerRegisters", _("Registers"),
                              NULL, ANJUTA_SHELL_PLACEMENT_LEFT,
 							 NULL);
 
+	return TRUE;
 }
 
 static void
 on_debugger_started (CpuRegisters *self)
 {
-	create_cpu_registers_gui (self);
+	if (!(dma_debugger_queue_get_feature (self->debugger) & HAS_CPU)) return;
+
+	/* If current debugger support access to cpu hardware */
+	if (!create_cpu_registers_gui (self)) return;
+
 	self->current_update = 0;
-}
-
-static void
-destroy_cpu_registers_gui (CpuRegisters *self)
-{
-	if (self->scrolledwindow != NULL)
-	{
-		dma_thread_clear_all_register_list (self);
-		gtk_widget_destroy (self->scrolledwindow);
-		self->scrolledwindow = NULL;
-	}
-}
-
-static void
-on_debugger_stopped (CpuRegisters *self)
-{
-	destroy_cpu_registers_gui (self);
-}
-
-static void
-on_frame_changed (CpuRegisters *self, guint frame, guint thread)
-{
-	dma_thread_set_register_list (self, thread);
+		
+	/* Connect remaining signal */
+	g_signal_connect_swapped (self->plugin, "debugger-stopped", G_CALLBACK (on_debugger_stopped), self);
+	g_signal_connect_swapped (self->plugin, "program-moved", G_CALLBACK (on_program_moved), self);
+	g_signal_connect_swapped (self->plugin, "frame-changed", G_CALLBACK (on_frame_changed), self);
 }
 
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
 CpuRegisters*
-cpu_registers_new(AnjutaPlugin *plugin, IAnjutaDebugger *debugger)
+cpu_registers_new(DebugManagerPlugin *plugin)
 {
 	CpuRegisters* self;
+
+	g_return_val_if_fail (plugin != NULL, NULL);
 	
 	self = g_new0 (CpuRegisters, 1);
 	
-	self->debugger = debugger;
-	if (debugger != NULL) g_object_ref (debugger);
-	self->plugin = plugin;
-	self->list = NULL;
+	self->plugin = ANJUTA_PLUGIN (plugin);
+	self->debugger = dma_debug_manager_get_queue (plugin);
 	
-	g_signal_connect_swapped (self->debugger, "debugger-started", G_CALLBACK (on_debugger_started), self);
-	g_signal_connect_swapped (self->debugger, "debugger-stopped", G_CALLBACK (on_debugger_stopped), self);
-	g_signal_connect_swapped (self->debugger, "program-stopped", G_CALLBACK (on_program_stopped), self);
-	g_signal_connect_swapped (self->debugger, "frame_changed", G_CALLBACK (on_frame_changed), self);
+	g_signal_connect_swapped (self->plugin, "debugger-started", G_CALLBACK (on_debugger_started), self);
 	
 	return self;
 }
@@ -489,14 +529,9 @@ cpu_registers_free(CpuRegisters* self)
 {
 	g_return_if_fail (self != NULL);
 
+	g_signal_handlers_disconnect_matched (self->plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);	
+	
 	destroy_cpu_registers_gui (self);
-	if (self->debugger != NULL)
-	{
-		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_debugger_started), self);
-		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_debugger_stopped), self);
-		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_program_stopped), self);
-		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_frame_changed), self);
-		g_object_unref (self->debugger);
-	}
+	
 	g_free(self);
 }
