@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -93,16 +93,3 @@ gdb_util_remove_blank_lines (const GList * lines)
 	}
 	return list;
 }
-
-void
-goto_location_in_editor (AnjutaPlugin *plugin, const gchar* uri, guint line)
-{
-	IAnjutaDocumentManager *docman;
-	
-	docman = anjuta_shell_get_interface (plugin->shell, IAnjutaDocumentManager, NULL);
-	if (docman)
-	{
-		
-		ianjuta_document_manager_goto_file_line (docman, uri, line, NULL);
-	}
-}
