@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifndef _UTILITIES_H_
 #define _UTILITIES_H_
@@ -34,8 +34,4 @@ gchar* gdb_util_remove_white_spaces(const gchar* text);
  *---------------------------------------------------------------------------*/
 GList* gdb_util_remove_blank_lines(const GList* lines);
 
-/* Goto to corresponding location in editor, opening file if necessary
- *---------------------------------------------------------------------------*/
-void goto_location_in_editor (AnjutaPlugin *plugin, const gchar* uri, guint line);
-
 #endif
