@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     watch.h
     Copyright (C) 2000  Kh. Naba Kumar Singh
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef _WATCH_H_
@@ -37,26 +38,7 @@ enum {
 	WATCH_N_COLUMNS
 };
 
-ExprWatch* expr_watch_new (AnjutaPlugin *plugin, IAnjutaDebugger *debugger);
-
-void expr_watch_connect (ExprWatch *ew, IAnjutaDebugger *debugger);
-
-/*void expr_watch_clear (ExprWatch *ew);
-
-void expr_watch_cmd_queqe (ExprWatch *ew);
-
-void expr_watch_update (Debugger *debugger, const GDBMIValue *mi_result,
-						const GList *cli_result, gpointer  ew);
-
-void expr_watch_evaluate_expression (ExprWatch *ew, const gchar *expr,
-									 DebuggerResultFunc parser, gpointer data);
-
-void expr_watch_update_controls (ExprWatch *ew);*/
-
+ExprWatch* expr_watch_new (AnjutaPlugin *plugin);
 void expr_watch_destroy (ExprWatch*ew);
 
-//void expr_watch_update (struct _DebugPrintCallback* cb_data, const gchar* value);
-void expr_watch_cmd_queqe (ExprWatch * ew);
-
-
 #endif
