@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #include <config.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -27,7 +27,6 @@
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-editor-convert.h>
 #include <libanjuta/interfaces/ianjuta-editor-line-mode.h>
-#include <libanjuta/interfaces/ianjuta-editor-autocomplete.h>
 #include <libanjuta/interfaces/ianjuta-editor-view.h>
 #include <libanjuta/interfaces/ianjuta-editor-folds.h>
 #include <libanjuta/interfaces/ianjuta-bookmark.h>
@@ -48,66 +47,14 @@
 #include "action-callbacks.h"
 #include "plugin.h"
 #include "file_history.h"
-#include "goto_line.h"
+#include "search-box.h"
 
-#define AUTOFORMAT_DISABLE         "autoformat.disable"
-#define AUTOFORMAT_STYLE           "autoformat.style"
-#define AUTOFORMAT_LIST_STYLE      "autoformat.list.style"
-#define AUTOFORMAT_OPTS            "autoformat.opts"
-
-void
-static editor_autoformat (IAnjutaEditor *te, AnjutaPreferences* prefs)
-{
-	gchar *cmd,*dir;
-	gchar *indent_style = NULL;
-	gchar *fopts = NULL;
-	pid_t pid;
-	int status;
-	const gchar* file;
-
-	if (anjuta_util_prog_is_installed ("indent", TRUE) == FALSE)
-		return;
-	if (anjuta_preferences_get_int (prefs, AUTOFORMAT_DISABLE))
-	{
-		GtkWindow *parent;
-		parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (te)));
-		anjuta_util_dialog_warning (parent, _("Auto format is currently disabled. Change the setting in Preferences."));
-		return;
-	}
-	if (te == NULL)
-		return;
-
-	file = ianjuta_editor_get_filename(IANJUTA_EDITOR(te), NULL);
-	
-	if (!anjuta_preferences_get_pair (prefs, AUTOFORMAT_STYLE,
-                                 GCONF_VALUE_STRING, GCONF_VALUE_STRING,
-                                 &indent_style, &fopts))
-		return;
-	
-	if (!fopts)
-	{
-		gchar *msg;
-		msg = g_strdup_printf (_("Anjuta does not know %s!"), indent_style);
-		anjuta_util_dialog_warning (NULL, msg);
-		g_free(msg);
-		return;
-	}
-	cmd = g_strconcat ("indent ", fopts, " ", file, NULL);
-	g_free (fopts);
-	dir = g_path_get_dirname (file);
-	pid = anjuta_util_execute_shell (dir, cmd);
-	g_free (dir);
-	g_free (cmd);
-
-	waitpid (pid, &status, 0);
-}
-
-static IAnjutaEditor*
+static IAnjutaDocument*
 get_current_editor(gpointer user_data)
 {
 	DocmanPlugin* plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	AnjutaDocman* docman = ANJUTA_DOCMAN(plugin->docman);
-	return IANJUTA_EDITOR(anjuta_docman_get_current_editor(docman));
+	return IANJUTA_DOCUMENT(anjuta_docman_get_current_document(docman));
 }
 
 void
@@ -123,17 +70,17 @@ on_open_activate (GtkAction * action, gpointer user_data)
 void
 on_save_activate (GtkAction *action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
 	
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	
-	anjuta_docman_save_editor (docman, te, NULL);
+	anjuta_docman_save_document (docman, te, NULL);
 }
 
 void
@@ -141,16 +88,16 @@ on_save_as_activate (GtkAction * action, gpointer user_data)
 {
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
 	
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	
-	anjuta_docman_save_editor_as (docman, te, NULL);
+	anjuta_docman_save_document_as (docman, te, NULL);
 }
 
 void
@@ -188,7 +135,7 @@ on_save_prompt_save_editor (AnjutaSavePrompt *save_prompt,
 	AnjutaDocman *docman;
 	
 	docman = ANJUTA_DOCMAN (user_data);
-	return anjuta_docman_save_editor (docman, IANJUTA_EDITOR (item),
+	return anjuta_docman_save_document (docman, IANJUTA_DOCUMENT (item),
 									  GTK_WIDGET (save_prompt));
 }
 
@@ -196,13 +143,13 @@ void
 on_close_file_activate (GtkAction * action, gpointer user_data)
 {
 	AnjutaDocman *docman;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	GtkWidget *parent;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	
@@ -217,7 +164,7 @@ on_close_file_activate (GtkAction * action, gpointer user_data)
 		save_prompt = anjuta_save_prompt_new (GTK_WINDOW (parent));
 		uri = ianjuta_file_get_uri (IANJUTA_FILE (te), NULL);
 		anjuta_save_prompt_add_item (save_prompt,
-									 ianjuta_editor_get_filename (te, NULL),
+									 ianjuta_document_get_filename (te, NULL),
 									 uri, te, on_save_prompt_save_editor,
 									 docman);
 		
@@ -229,13 +176,13 @@ on_close_file_activate (GtkAction * action, gpointer user_data)
 			case ANJUTA_SAVE_PROMPT_RESPONSE_DISCARD:
 			case ANJUTA_SAVE_PROMPT_RESPONSE_SAVE_CLOSE:
 				/* Close it */
-				anjuta_docman_remove_editor (docman, te);
+				anjuta_docman_remove_document (docman, te);
 				break;
 		}
 		gtk_widget_destroy (GTK_WIDGET (save_prompt));
 	}
 	else
-		anjuta_docman_remove_editor (docman, te);
+		anjuta_docman_remove_document (docman, te);
 }
 
 void
@@ -259,17 +206,28 @@ on_close_all_file_activate (GtkAction * action, gpointer user_data)
 		{
 			if (!ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE (te), NULL))
 			{
-				anjuta_docman_remove_editor (docman, te);
+				anjuta_docman_remove_document (docman, IANJUTA_DOCUMENT(te));
 			}
 		}
 		node = next;
 	}
 }
 
+void
+on_new_file_activate (GtkAction *action, gpointer user_data)
+{
+	AnjutaDocman *docman;
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	docman = ANJUTA_DOCMAN (plugin->docman);
+
+	anjuta_docman_add_editor (docman, NULL, NULL);
+}
+
 void
 on_reload_file_activate (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	gchar mesg[256];
 	GtkWidget *dialog;
 	GtkWidget *parent;
@@ -278,13 +236,13 @@ on_reload_file_activate (GtkAction * action, gpointer user_data)
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 
 	sprintf (mesg, _("Are you sure you want to reload '%s'?\n"
 					 "Any unsaved changes will be lost."),
-			 ianjuta_editor_get_filename(te, NULL));
+			 ianjuta_document_get_filename(te, NULL));
 
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (te));
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
@@ -310,13 +268,13 @@ on_reload_file_activate (GtkAction * action, gpointer user_data)
 void
 anjuta_print_cb (GtkAction *action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	ianjuta_print_print (IANJUTA_PRINT(te), NULL);
@@ -325,13 +283,13 @@ anjuta_print_cb (GtkAction *action, gpointer user_data)
 void
 anjuta_print_preview_cb (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	ianjuta_print_print_preview (IANJUTA_PRINT(te), NULL);
@@ -340,25 +298,29 @@ anjuta_print_preview_cb (GtkAction * action, gpointer user_data)
 void
 on_editor_command_upper_case_activate (GtkAction * action, gpointer data)
 {
-	IAnjutaEditor *editor;
+	IAnjutaDocument *editor;
 	gint start, end;
 	
 	editor = get_current_editor (data);
 	start = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor), NULL);
 	end = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor), NULL);
-	ianjuta_editor_convert_to_upper (IANJUTA_EDITOR_CONVERT (editor), start, end, NULL);
+	if (start >= 0 && end >= 0)
+		ianjuta_editor_convert_to_upper (IANJUTA_EDITOR_CONVERT (editor),
+										 start, end, NULL);
 }
 
 void
 on_editor_command_lower_case_activate (GtkAction * action, gpointer data)
 {
-	IAnjutaEditor *editor;
+	IAnjutaDocument *editor;
 	gint start, end;
 	
 	editor = get_current_editor (data);
 	start = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor), NULL);
 	end = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor), NULL);
-	ianjuta_editor_convert_to_lower (IANJUTA_EDITOR_CONVERT (editor), start, end, NULL);
+	if (start >= 0 && end >= 0)
+		ianjuta_editor_convert_to_lower (IANJUTA_EDITOR_CONVERT (editor),
+										 start, end, NULL);
 }
 
 void
@@ -423,13 +385,13 @@ on_editor_command_match_brace_activate (GtkAction * action, gpointer data)
 void
 on_editor_command_undo_activate (GtkAction * action, gpointer data)
 {
-	ianjuta_editor_undo (get_current_editor(data), NULL);
+	ianjuta_document_undo (get_current_editor(data), NULL);
 }
 
 void
 on_editor_command_redo_activate (GtkAction * action, gpointer data)
 {
-	ianjuta_editor_redo (get_current_editor(data), NULL);
+	ianjuta_document_redo (get_current_editor(data), NULL);
 }
 
 void
@@ -447,7 +409,7 @@ on_editor_command_cut_activate (GtkAction * action, gpointer data)
 	}
 	else
 	{
-		ianjuta_editor_selection_cut (IANJUTA_EDITOR_SELECTION (get_current_editor(data)), NULL);
+		ianjuta_document_cut (IANJUTA_DOCUMENT (get_current_editor(data)), NULL);
 	}
 }
 
@@ -466,7 +428,7 @@ on_editor_command_copy_activate (GtkAction * action, gpointer data)
 	}
 	else
 	{
-		ianjuta_editor_selection_copy (IANJUTA_EDITOR_SELECTION (get_current_editor(data)), NULL);
+		ianjuta_document_copy (IANJUTA_DOCUMENT (get_current_editor(data)), NULL);
 	}
 }
 
@@ -485,7 +447,7 @@ on_editor_command_paste_activate (GtkAction * action, gpointer data)
 	}
 	else
 	{
-		ianjuta_editor_selection_paste (IANJUTA_EDITOR_SELECTION (get_current_editor(data)), NULL);
+		ianjuta_document_paste (IANJUTA_DOCUMENT (get_current_editor(data)), NULL);
 	}
 }
 
@@ -513,16 +475,10 @@ on_editor_command_clear_activate (GtkAction * action, gpointer data)
 	}
 	else
 	{
-		ianjuta_editor_selection_clear (IANJUTA_EDITOR_SELECTION (get_current_editor(data)), NULL);
+		ianjuta_document_clear (IANJUTA_DOCUMENT (get_current_editor(data)), NULL);
 	}
 }
 
-void
-on_editor_command_complete_word_activate (GtkAction * action, gpointer data)
-{
-	ianjuta_editor_autocomplete_activate (IANJUTA_EDITOR_AUTOCOMPLETE (get_current_editor(data)), NULL);
-}
-
 void on_editor_command_close_folds_all_activate (GtkAction * action, gpointer data)
 {
 	ianjuta_editor_folds_close_all (IANJUTA_EDITOR_FOLDS (get_current_editor(data)), NULL);
@@ -541,7 +497,7 @@ void on_editor_command_toggle_fold_activate (GtkAction * action, gpointer data)
 void on_editor_command_bookmark_toggle_activate (GtkAction * action, gpointer data)
 {
 	ianjuta_bookmark_toggle (IANJUTA_BOOKMARK (get_current_editor (data)),
-							 ianjuta_editor_get_lineno (get_current_editor (data), NULL),
+							 ianjuta_editor_get_lineno (IANJUTA_EDITOR(get_current_editor (data)), NULL),
 							 FALSE, NULL);
 }
 
@@ -600,122 +556,64 @@ on_next_history (GtkAction *action, gpointer user_data)
 void
 on_comment_block (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor* te;
+	IAnjutaDocument* te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document(docman);
 	ianjuta_editor_comment_block(IANJUTA_EDITOR_COMMENT(te), NULL);
 }
 
 void on_comment_box (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor* te;
+	IAnjutaDocument* te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	ianjuta_editor_comment_box(IANJUTA_EDITOR_COMMENT(te), NULL);
 }
 
 void on_comment_stream (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor* te;
+	IAnjutaDocument* te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	ianjuta_editor_comment_stream(IANJUTA_EDITOR_COMMENT(te), NULL);
 }
 
-void
-on_goto_activate (GtkAction *action, gpointer user_data)
-{
-	DocmanPlugin *plugin;
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	anjuta_ui_activate_action_by_path (plugin->ui,
-		"ActionGroupNavigation/ActionEditGotoLineEntry");
-}
-
-void
-on_toolbar_goto_clicked (GtkAction *action, gpointer user_data)
-{
-	DocmanPlugin *plugin;
-	AnjutaUI *ui;
-	AnjutaShell* shell;
-	AnjutaDocman *docman;
-	IAnjutaEditor *te;
-	guint line;
-	const gchar *line_ascii;
-	
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
-	g_object_get(G_OBJECT(plugin), "shell", &shell, NULL);
-	ui = anjuta_shell_get_ui(shell, NULL);
-	
-	if (EGG_IS_ENTRY_ACTION (action))
-	{
-		line_ascii = egg_entry_action_get_text (EGG_ENTRY_ACTION (action));
-	}
-	else
-	{
-		GtkAction *entry_action;
-		entry_action = anjuta_ui_get_action (ui, "ActionNavigation",
-									   "ActionEditSearchEntry");
-		g_return_if_fail (EGG_IS_ENTRY_ACTION (action));
-		line_ascii = egg_entry_action_get_text (EGG_ENTRY_ACTION (action));
-	}
-	if (strlen (line_ascii) == 0)
-		return;
-	
-	if (te)
-	{
-		line = atoi (line_ascii);
-		ianjuta_editor_goto_line(te, line, NULL);
-		if (IANJUTA_IS_MARKABLE (te))
-		{
-			ianjuta_markable_delete_all_markers (IANJUTA_MARKABLE (te),
-												 IANJUTA_MARKABLE_LINEMARKER,
-												 NULL);
-			ianjuta_markable_mark (IANJUTA_MARKABLE (te),
-								   line, IANJUTA_MARKABLE_LINEMARKER, NULL);
-		}
-	}
-}
-
 void
 on_goto_line_no1_activate (GtkAction * action, gpointer user_data)
 {
-	GtkWidget *gt;
 	DocmanPlugin *plugin;
 	AnjutaDocman *docman;
-	IAnjutaEditor *te;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
 	
-	if (te)
+	if (!gtk_widget_get_parent (plugin->search_box))
 	{
-		gt = gotoline_new (te);
-		gtk_widget_show (gt);
+			gtk_box_pack_end (GTK_BOX(plugin->vbox), plugin->search_box, FALSE, FALSE, 0);
 	}
+	gtk_widget_show (plugin->search_box);
+	search_box_grab_line_focus (SEARCH_BOX (plugin->search_box));
 }
 
 void
 on_goto_block_start1_activate (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document(docman);
 	if (te == NULL)
 		return;
 	ianjuta_editor_goto_start_block(IANJUTA_EDITOR_GOTO(te), NULL);
@@ -724,13 +622,13 @@ on_goto_block_start1_activate (GtkAction * action, gpointer user_data)
 void
 on_goto_block_end1_activate (GtkAction * action, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (te == NULL)
 		return;
 	ianjuta_editor_goto_end_block(IANJUTA_EDITOR_GOTO(te), NULL);
@@ -826,11 +724,11 @@ on_zoom_in_text_activate (GtkAction * action, gpointer user_data)
 {
 	DocmanPlugin *plugin;
 	AnjutaDocman *docman;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	
 	if (te == NULL)
 		return;
@@ -842,11 +740,11 @@ on_zoom_out_text_activate (GtkAction * action, gpointer user_data)
 {
 	DocmanPlugin *plugin;
 	AnjutaDocman *docman;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	
 	if (te == NULL)
 		return;
@@ -856,14 +754,14 @@ on_zoom_out_text_activate (GtkAction * action, gpointer user_data)
 void
 on_force_hilite_activate (GtkWidget *menuitem, gpointer user_data)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	const gchar *language_code;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document(docman);
 	language_code = g_object_get_data (G_OBJECT (menuitem), "language_code");
 	
 	if (te == NULL)
@@ -873,41 +771,6 @@ on_force_hilite_activate (GtkWidget *menuitem, gpointer user_data)
 											  language_code, NULL);
 }
 
-void
-on_indent1_activate (GtkAction * action, gpointer user_data)
-{
-	//trying to restore line no where i was before autoformat invoked
-	gint lineno;
-	IAnjutaEditor *te;
-	AnjutaDocman *docman;
-	DocmanPlugin *plugin;
-	
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	docman = ANJUTA_DOCMAN (plugin->docman);
-	te = anjuta_docman_get_current_editor (docman);
-	if (te == NULL)
-		return;
-	lineno = ianjuta_editor_get_lineno(te, NULL);
-	editor_autoformat (te, plugin->prefs);
-	ianjuta_editor_goto_line (te, lineno, NULL);
-}
-
-void
-on_format_indent_style_clicked (GtkAction * action, gpointer user_data)
-{
-	DocmanPlugin *plugin;
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	gtk_signal_emit_by_name (GTK_OBJECT (plugin->prefs),
-										 "activate");
-}
-
-void
-on_calltip1_activate (GtkAction * action, gpointer user_data)
-{
-	// FIXME
-	// on_editor_command_activate (action, ANE_COMPLETECALLTIP, user_data);
-}
-
 /* Gets the swapped (c/h) file names */
 static gchar*
 get_swapped_filename (const gchar* filename)
@@ -1013,14 +876,14 @@ on_swap_activate (GtkAction *action, gpointer user_data)
 {
 	gchar *newfname;
 	const gchar *uri;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
 	
-	te = anjuta_docman_get_current_editor (docman);
+	te = anjuta_docman_get_current_document (docman);
 	if (!te)
 		return;
 	uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
@@ -1035,10 +898,26 @@ on_swap_activate (GtkAction *action, gpointer user_data)
 	return;
 }
 
+void
+on_show_search (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	
+	if (!gtk_widget_get_parent (plugin->search_box))
+	{
+			gtk_box_pack_end (GTK_BOX(plugin->vbox), plugin->search_box, FALSE, FALSE, 0);
+	}
+
+	gtk_widget_show (plugin->search_box);
+	search_box_grab_search_focus (SEARCH_BOX (plugin->search_box));
+}
+
 void
 on_editor_add_view_activate (GtkAction *action, gpointer user_data)
 {
-	IAnjutaEditor *te = get_current_editor (user_data);
+	IAnjutaDocument *te = get_current_editor (user_data);
 	if (!te)
 		return;
 	ianjuta_editor_view_create (IANJUTA_EDITOR_VIEW (te), NULL);
@@ -1047,7 +926,7 @@ on_editor_add_view_activate (GtkAction *action, gpointer user_data)
 void
 on_editor_remove_view_activate (GtkAction *action, gpointer user_data)
 {
-	IAnjutaEditor *te = get_current_editor (user_data);
+	IAnjutaDocument *te = get_current_editor (user_data);
 	if (!te)
 		return;
 	ianjuta_editor_view_remove_current (IANJUTA_EDITOR_VIEW (te), NULL);
