@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -23,6 +23,7 @@
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-iterable.h>
+#include <libanjuta/interfaces/ianjuta-document.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-cell.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
@@ -30,9 +31,17 @@
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-preferences.h>
 #include <libanjuta/interfaces/ianjuta-symbol.h>
-
+#include <libanjuta/interfaces/ianjuta-language.h>
 
 #include "plugin.h"
+#include "cpp-java-utils.h"
+
+#define ANJUTA_PIXMAP_COMPLETE			  "anjuta-complete"
+#define ANJUTA_PIXMAP_AUTOCOMPLETE        "anjuta-complete-auto"
+#define ANJUTA_PIXMAP_AUTOINDENT          "anjuta-indent-auto"
+#define ANJUTA_STOCK_COMPLETE         	  "anjuta-complete"
+#define ANJUTA_STOCK_AUTOCOMPLETE         "anjuta-autocomplete"
+#define ANJUTA_STOCK_AUTOINDENT           "anjuta-indent"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-language-support-cpp-java.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-language-cpp-java.glade"
@@ -63,7 +72,6 @@
 #define CASE_INDENT (INDENT_SIZE)
 #define LABEL_INDENT (INDENT_SIZE)
 
-#define LEFT_BRACE(ch) (ch == ')'? '(' : (ch == '}'? '{' : (ch == ']'? '[' : ch)))  
 static gpointer parent_class;
 
 static gboolean
@@ -124,52 +132,6 @@ skip_iter_to_newline_tail (IAnjutaIterable *iter, gchar ch)
 
 /* Jumps to the reverse matching brace of the given brace character */
 
-static gboolean
-jumb_to_matching_brace (IAnjutaIterable *iter, gchar brace)
-{
-	gchar point_ch = brace;
-	GString *braces_stack = g_string_new ("");
-	
-	g_return_val_if_fail (point_ch == ')' || point_ch == ']' ||
-						  point_ch == '}', FALSE);
-	
-	/* DEBUG_PRINT ("Matching brace being"); */
-	/* Push brace */
-	g_string_prepend_c (braces_stack, point_ch);
-	
-	while (ianjuta_iterable_previous (iter, NULL))
-	{
-		/* Skip comments and strings */
-		IAnjutaEditorAttribute attrib =
-			ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter), NULL);
-		if (attrib == IANJUTA_EDITOR_COMMENT || attrib == IANJUTA_EDITOR_STRING)
-			continue;
-		
-		/* DEBUG_PRINT ("point ch = %c", point_ch); */
-		point_ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0,
-												 NULL);
-		if (point_ch == ')' || point_ch == ']' || point_ch == '}')
-		{
-			/* Push brace */
-			g_string_prepend_c (braces_stack, point_ch);
-			continue;
-		}
-		if (point_ch == LEFT_BRACE (braces_stack->str[0]))
-		{
-			/* Pop brace */
-			g_string_erase (braces_stack, 0, 1);
-		}
-		/* Bail out if there is no more in stack */
-		if (braces_stack->str[0] == '\0')
-		{
-			/* DEBUG_PRINT ("Matching brace end -- found"); */
-			return TRUE;
-		}
-	}
-	/* DEBUG_PRINT ("Matching brace end -- not found"); */
-	return FALSE;
-}
-
 static gint
 get_line_indentation (IAnjutaEditor *editor, gint line_num)
 {
@@ -607,223 +569,7 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 		}
 	}
 	g_string_free (comment_text, TRUE);
-}
-
-static void on_assist_begin(IAnjutaEditorAssist* assist, gchar* context, gchar* trigger, CppJavaPlugin* lang_plugin)
-{
-	gint position;
-	
-	DEBUG_PRINT("assist-begin: %s", context);
-	/* General word completion */
-	if (context == NULL || !strlen(context))
-	{
-		ianjuta_editor_assist_suggest(assist, NULL, -1, NULL);
-		return;
-	}
-	if (!trigger)
-	{
-		IAnjutaIterable* iter = 
-			ianjuta_symbol_manager_search (lang_plugin->symbol_browser,
-										   IANJUTA_SYMBOL_TYPE_FUNCTION|
-										   IANJUTA_SYMBOL_TYPE_CLASS|
-										   IANJUTA_SYMBOL_TYPE_ENUM,
-											context, TRUE, TRUE, NULL);
-		if (iter)
-		{
-			do
-			{
-				const gchar* name = ianjuta_symbol_name(IANJUTA_SYMBOL(iter), NULL);
-				if (name != NULL)
-				{
-					lang_plugin->suggestions = g_list_append(lang_plugin->suggestions,
-													 g_strdup(name));
-				}
-				else
-					break;
-			}
-			while (ianjuta_iterable_next(iter, NULL));
-			position = ianjuta_editor_get_position(IANJUTA_EDITOR(assist), NULL)
-				- strlen(context);
-		}
-	}
-	else if (g_str_equal(trigger, "::"))
-	{
-		IAnjutaIterable* iter = 
-			ianjuta_symbol_manager_get_members (lang_plugin->symbol_browser,
-											context, TRUE, NULL);
-		if (iter)
-		{
-			do
-			{
-				const gchar* name = ianjuta_symbol_name(IANJUTA_SYMBOL(iter), NULL);
-				if (name != NULL)
-				{
-					lang_plugin->suggestions = g_list_append(lang_plugin->suggestions,
-													 g_strdup(name));
-				}
-			}
-			while (ianjuta_iterable_next(iter, NULL));
-		}
-		position = ianjuta_editor_get_position(IANJUTA_EDITOR(assist), NULL);
-	}
-	else if (g_str_equal(trigger, ".") || g_str_equal(trigger, "->"))
-	{
-		/* TODO: Find the type of context by parsing the file somehow and
-		search for the member as it is done with the :: context */
-	}
-	/* More than 100 suggestions are likely to break X Windows...
-	We better wait until more context arrives
-	TODO: Make user configurable */
-	if (g_list_length(lang_plugin->suggestions) < 100)
-		ianjuta_editor_assist_suggest(assist, lang_plugin->suggestions, position, NULL);
-}
-
-static void assist_cleanup(CppJavaPlugin* lang_plugin)
-{
-	g_list_foreach(lang_plugin->suggestions, (GFunc) g_free, NULL);
-	g_list_free(lang_plugin->suggestions);
-	lang_plugin->suggestions = NULL;
-}
-
-static void on_assist_end(IAnjutaEditorAssist* assist, CppJavaPlugin* lang_plugin)
-{
-	DEBUG_PRINT("assist-end");
-	assist_cleanup(lang_plugin);
-}
-
-static void on_assist_cancel(IAnjutaEditorAssist* assist, CppJavaPlugin* lang_plugin)
-{
-	DEBUG_PRINT("assist-cancel");
-	assist_cleanup(lang_plugin);
-}
-
-static void on_assist_chosen(IAnjutaEditorAssist* assist, gint selection, CppJavaPlugin* lang_plugin)
-{
-	DEBUG_PRINT("assist-chosen: %d", selection);
-	gchar* assistance = g_list_nth_data(lang_plugin->suggestions, selection);
-	ianjuta_editor_assist_react(assist, selection, assistance, NULL);  
-	assist_cleanup(lang_plugin);
-}
-
-static void on_assist_update(IAnjutaEditorAssist* assist, gchar* context, CppJavaPlugin* lang_plugin)
-{
-	DEBUG_PRINT("assist-update: %s", context);
-	GList* suggestions = NULL;
-	GList* sug;
-	if (!strlen(context))
-	{
-		ianjuta_editor_assist_suggest(assist, NULL, -1, NULL);
-		return;
-	}
-	for (sug = lang_plugin->suggestions; sug != NULL; sug = sug->next)
-	{
-		gchar* suggestion = sug->data;
-		if (g_str_has_prefix(suggestion, context))
-		{
-			suggestions = g_list_append(suggestions, g_strdup(suggestion));
-		}
-	}
-	assist_cleanup(lang_plugin);
-	lang_plugin->suggestions = suggestions;
-	ianjuta_editor_assist_suggest(assist, lang_plugin->suggestions, -1, NULL);
-}
-
-static gchar*
-get_context(IAnjutaEditorCell* iter, IAnjutaEditor* editor)
-{
-	gint end;
-	gint begin;
-	gchar* utf8 = NULL;
-	gunichar unichar;
-	end = ianjuta_iterable_get_position(IANJUTA_ITERABLE(iter), NULL);
-	do
-	{
-		ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL);		
-		utf8 = ianjuta_editor_cell_get_character(iter, NULL);
-		unichar = g_utf8_get_char(utf8);
-		g_free(utf8);
-	}
-	while (!g_unichar_isspace(unichar));
-	begin = ianjuta_iterable_get_position(IANJUTA_ITERABLE(iter), NULL) + 1;
-	return ianjuta_editor_get_text(editor, begin, end - begin, NULL);
-}
-
-static gchar*
-dot_member_parser(IAnjutaEditor* editor, gint position)
-{
-	IAnjutaEditorCell* iter = IANJUTA_EDITOR_CELL(ianjuta_editor_get_cell_iter(editor, position, NULL));
-	if (ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL) &&
-		ianjuta_editor_cell_get_char(iter, 0, NULL) == '.')
-	{
-		return get_context(iter, editor);
-	}
-	else
-	{
-		return NULL;
-	}
-}
-
-static gchar*
-pointer_member_parser(IAnjutaEditor* editor, gint position)
-{
-	IAnjutaEditorCell* iter = IANJUTA_EDITOR_CELL(ianjuta_editor_get_cell_iter(editor, position, NULL));
-	if (ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL) &&
-		ianjuta_editor_cell_get_char(iter, 0, NULL) == '>' &&
-		ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL) &&
-		ianjuta_editor_cell_get_char(iter, 0, NULL) == '-')
-	{
-		return get_context(iter, editor);
-	}
-	else
-	{
-		return NULL;
-	}
-}
-
-static gchar*
-cpp_member_parser(IAnjutaEditor* editor, gint position)
-{
-	IAnjutaEditorCell* iter = IANJUTA_EDITOR_CELL(ianjuta_editor_get_cell_iter(editor, position, NULL));
-	if (ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL) &&
-		ianjuta_editor_cell_get_char(iter, 0, NULL) == ':' &&
-		ianjuta_iterable_previous(IANJUTA_ITERABLE(iter), NULL) &&
-		ianjuta_editor_cell_get_char(iter, 0, NULL) == ':')
-	{
-		return get_context(iter, editor);
-	}
-	else
-	{
-		return NULL;
-	}
-}
-
-
-static void
-install_assist(CppJavaPlugin* lang_plugin)
-{
-	IAnjutaEditorAssist* assist = IANJUTA_EDITOR_ASSIST(lang_plugin->current_editor);
-	ianjuta_editor_assist_add_trigger(assist, ".", dot_member_parser, NULL);
-	ianjuta_editor_assist_add_trigger(assist, "->", pointer_member_parser, NULL);
-	ianjuta_editor_assist_add_trigger(assist, "::", cpp_member_parser, NULL);	
-	g_signal_connect(G_OBJECT(assist), "assist_begin", G_CALLBACK(on_assist_begin), lang_plugin);
-	g_signal_connect(G_OBJECT(assist), "assist_end", G_CALLBACK(on_assist_end), lang_plugin);
-	g_signal_connect(G_OBJECT(assist), "assist_update", G_CALLBACK(on_assist_update), lang_plugin);
-	g_signal_connect(G_OBJECT(assist), "assist_chosen", G_CALLBACK(on_assist_chosen), lang_plugin);
-	g_signal_connect(G_OBJECT(assist), "assist_canceled", G_CALLBACK(on_assist_cancel), lang_plugin);
-}
-
-static void
-uninstall_assist(CppJavaPlugin* lang_plugin)
-{
-	IAnjutaEditorAssist* assist = IANJUTA_EDITOR_ASSIST(lang_plugin->current_editor);
-	ianjuta_editor_assist_remove_trigger(assist, ".", NULL);
-	ianjuta_editor_assist_remove_trigger(assist, "->", NULL);
-	ianjuta_editor_assist_remove_trigger(assist, "::", NULL);
-	g_signal_handlers_disconnect_by_func(assist, G_CALLBACK(on_assist_begin), lang_plugin);
-	g_signal_handlers_disconnect_by_func(assist, G_CALLBACK(on_assist_end), lang_plugin);
-	g_signal_handlers_disconnect_by_func(assist, G_CALLBACK(on_assist_update), lang_plugin);
-	g_signal_handlers_disconnect_by_func(assist, G_CALLBACK(on_assist_cancel), lang_plugin);
-	g_signal_handlers_disconnect_by_func(assist, G_CALLBACK(on_assist_chosen), lang_plugin);
+	g_object_unref (iter);
 }
 
 static gint
@@ -1039,7 +785,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			}
 			
 			/* Find matching brace and continue */
-			if (!jumb_to_matching_brace (iter, point_ch))
+			if (!cpp_java_util_jump_to_matching_brace (iter, point_ch, -1))
 			{
 				line_indent = get_line_indentation (editor, line_saved);
 				break;
@@ -1238,7 +984,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		else if (ch == '}')
 		{
 			ianjuta_iterable_previous (iter, NULL);
-			if (jumb_to_matching_brace (iter, ch))
+			if (cpp_java_util_jump_to_matching_brace (iter, ch, -1))
 			{
 				gint position = ianjuta_iterable_get_position (iter, NULL);
 				gint line = ianjuta_editor_get_line_from_position (editor,
@@ -1353,12 +1099,12 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 		gint insert_line;
 		gint line_indent;
 		
-		ianjuta_editor_begin_undo_action (editor, NULL);
+		ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 		initialize_indentation_params (plugin);
 		insert_line = ianjuta_editor_get_lineno (editor, NULL);
 		line_indent = get_line_auto_indentation (plugin, editor, insert_line);
 		set_line_indentation (editor, insert_line, line_indent);
-		ianjuta_editor_end_undo_action (editor, NULL);
+		ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 	}
 	g_object_unref (iter);
 }
@@ -1374,46 +1120,76 @@ on_editor_char_inserted_java (IAnjutaEditor *editor,
 
 static void
 install_support (CppJavaPlugin *lang_plugin)
-{
+{	
+	IAnjutaLanguage* lang_manager =
+		anjuta_shell_get_interface (ANJUTA_PLUGIN (lang_plugin)->shell,
+									IAnjutaLanguage, NULL);
+	
+	if (!lang_manager)
+		return;
+	
 	if (lang_plugin->support_installed)
 		return;
 	
-	lang_plugin->current_language
-		= ianjuta_editor_language_get_language
-					(IANJUTA_EDITOR_LANGUAGE (lang_plugin->current_editor),
-											  NULL);
+	lang_plugin->current_language = 
+		ianjuta_language_get_name_from_editor (lang_manager, 
+											   IANJUTA_EDITOR_LANGUAGE (lang_plugin->current_editor), NULL);
 	
-	DEBUG_PRINT("Language: %s", lang_plugin->current_language);
+	DEBUG_PRINT("Language support intalled for: %s",
+				lang_plugin->current_language);
 	
 	if (lang_plugin->current_language &&
-		(strcasecmp (lang_plugin->current_language, "cpp") == 0
-		|| strcasecmp (lang_plugin->current_language, "c") == 0
-		|| strcasecmp (lang_plugin->current_language, "c++") == 0))
+		(g_str_equal (lang_plugin->current_language, "C")
+		|| g_str_equal (lang_plugin->current_language, "C++")))
 	{
 		g_signal_connect (lang_plugin->current_editor,
 						  "char-added",
 						  G_CALLBACK (on_editor_char_inserted_cpp),
 						  lang_plugin);
-		if (IANJUTA_IS_EDITOR_ASSIST(lang_plugin->current_editor))
-			install_assist(lang_plugin);
-		initialize_indentation_params (lang_plugin);
 	}
 	else if (lang_plugin->current_language &&
-		(strcasecmp (lang_plugin->current_language, "java") == 0
-		|| strcasecmp (lang_plugin->current_language, "Java") == 0))
+		(g_str_equal (lang_plugin->current_language, "Java")))
 	{
 		g_signal_connect (lang_plugin->current_editor,
 						  "char-added",
 						  G_CALLBACK (on_editor_char_inserted_java),
 						  lang_plugin);
-		initialize_indentation_params (lang_plugin);
 	}
 	else
 	{
 		return;
 	}
+	
+	initialize_indentation_params (lang_plugin);
+	if (IANJUTA_IS_EDITOR_ASSIST (lang_plugin->current_editor))
+	{
+		AnjutaPlugin *plugin;
+		AnjutaUI *ui;
+		GtkAction *action;
+		IAnjutaEditorAssist* iassist;
+		
+		plugin = ANJUTA_PLUGIN (lang_plugin);
+		ui = anjuta_shell_get_ui (plugin->shell, NULL);
+		iassist = IANJUTA_EDITOR_ASSIST (lang_plugin->current_editor);
+		
+		g_assert (lang_plugin->assist == NULL);
+		
+		lang_plugin->assist =
+			cpp_java_assist_new (iassist,
+				anjuta_shell_get_interface (plugin->shell,
+											IAnjutaSymbolManager,
+											NULL),
+								 lang_plugin->prefs);
+		
+		/* Enable autocompletion action */
+		action = gtk_action_group_get_action (lang_plugin->action_group, 
+									   "ActionEditAutocomplete");
+		g_object_set (G_OBJECT (action), "visible", TRUE,
+					  "sensitive", TRUE, NULL);
+	}
 	/* Disable editor intern auto-indent */
-	ianjuta_editor_set_auto_indent(IANJUTA_EDITOR(lang_plugin->current_editor), FALSE, NULL);
+	ianjuta_editor_set_auto_indent (IANJUTA_EDITOR(lang_plugin->current_editor),
+								    FALSE, NULL);
 	lang_plugin->support_installed = TRUE;
 }
 
@@ -1424,19 +1200,39 @@ uninstall_support (CppJavaPlugin *lang_plugin)
 		return;
 	
 	if (lang_plugin->current_language &&
-		strcmp (lang_plugin->current_language, "cpp") == 0)
+		(g_str_equal (lang_plugin->current_language, "C")
+		|| g_str_equal (lang_plugin->current_language, "C++")))
 	{
 		g_signal_handlers_disconnect_by_func (lang_plugin->current_editor,
 									G_CALLBACK (on_editor_char_inserted_cpp),
 									lang_plugin);
 	}
 	else if (lang_plugin->current_language &&
-		strcmp (lang_plugin->current_language, "java") == 0)
+		(g_str_equal (lang_plugin->current_language, "Java")))
 	{
 		g_signal_handlers_disconnect_by_func (lang_plugin->current_editor,
 									G_CALLBACK (on_editor_char_inserted_java),
 									lang_plugin);
 	}
+	
+	if (lang_plugin->assist)
+	{
+		AnjutaPlugin *plugin;
+		AnjutaUI *ui;
+		GtkAction *action;
+		
+		g_object_unref (lang_plugin->assist);
+		lang_plugin->assist = NULL;
+		
+		/* Disable autocompletion action */
+		plugin = ANJUTA_PLUGIN (lang_plugin);
+		ui = anjuta_shell_get_ui (plugin->shell, NULL);
+		action = gtk_action_group_get_action (lang_plugin->action_group,
+									   "ActionEditAutocomplete");
+		g_object_set (G_OBJECT (action), "visible", FALSE,
+					  "sensitive", FALSE, NULL);
+	}
+	
 	lang_plugin->support_installed = FALSE;
 }
 
@@ -1454,11 +1250,17 @@ on_value_added_current_editor (AnjutaPlugin *plugin, const gchar *name,
 							   const GValue *value, gpointer data)
 {
 	CppJavaPlugin *lang_plugin;
+	IAnjutaDocument* doc = IANJUTA_DOCUMENT(g_value_get_object (value));
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (plugin);
-	lang_plugin->current_editor = g_value_get_object (value);
-	install_support (lang_plugin);
-	if (IANJUTA_IS_EDITOR_ASSIST(lang_plugin->current_editor))
-		install_assist(lang_plugin);
+	if (IANJUTA_IS_EDITOR(doc))
+		lang_plugin->current_editor = G_OBJECT(doc);
+	else
+	{
+		lang_plugin->current_editor = NULL;
+		return;
+	}
+	if (IANJUTA_IS_EDITOR(lang_plugin->current_editor))
+		install_support (lang_plugin);
 	g_signal_connect (lang_plugin->current_editor, "language-changed",
 					  G_CALLBACK (on_editor_language_changed),
 					  plugin);
@@ -1470,12 +1272,12 @@ on_value_removed_current_editor (AnjutaPlugin *plugin, const gchar *name,
 {
 	CppJavaPlugin *lang_plugin;
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (plugin);
-	g_signal_handlers_disconnect_by_func (lang_plugin->current_editor,
+	if (lang_plugin->current_editor)
+		g_signal_handlers_disconnect_by_func (lang_plugin->current_editor,
 										  G_CALLBACK (on_editor_language_changed),
 										  plugin);
-	if (IANJUTA_IS_EDITOR_ASSIST(lang_plugin->current_editor))
-		uninstall_assist(lang_plugin);
-	uninstall_support (lang_plugin);
+	if (IANJUTA_IS_EDITOR(lang_plugin->current_editor))
+		uninstall_support (lang_plugin);
 	lang_plugin->current_editor = NULL;
 }
 
@@ -1486,23 +1288,34 @@ on_auto_indent (GtkAction *action, gpointer data)
 	gint line_start, line_end;
 	gint insert_line;
 	gint line_indent;
+	gboolean has_selection;
 	
 	CppJavaPlugin *lang_plugin;
 	IAnjutaEditor *editor;
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 	editor = IANJUTA_EDITOR (lang_plugin->current_editor);
 	
-	sel_start = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor),
-													NULL);
-	sel_end = ianjuta_editor_selection_get_end (IANJUTA_EDITOR_SELECTION (editor),
-												NULL);
-	line_start = ianjuta_editor_get_line_from_position (editor,
-														sel_start,
+	has_selection = ianjuta_editor_selection_has_selection
+						(IANJUTA_EDITOR_SELECTION (editor), NULL);
+	if (has_selection)
+	{
+		sel_start = ianjuta_editor_selection_get_start (IANJUTA_EDITOR_SELECTION (editor),
 														NULL);
-	line_end = ianjuta_editor_get_line_from_position (editor,
-													  sel_end,
-													  NULL);
-	ianjuta_editor_begin_undo_action (editor, NULL);
+		sel_end = ianjuta_editor_selection_get_end (IANJUTA_EDITOR_SELECTION (editor),
+													NULL);
+		line_start = ianjuta_editor_get_line_from_position (editor,
+															sel_start,
+															NULL);
+		line_end = ianjuta_editor_get_line_from_position (editor,
+														  sel_end,
+														  NULL);
+	}
+	else
+	{
+		line_start = ianjuta_editor_get_lineno (IANJUTA_EDITOR(editor), NULL);
+		line_end = line_start;
+	}
+	ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 	initialize_indentation_params (lang_plugin);
 	
 	for (insert_line = line_start; insert_line <= line_end; insert_line++)
@@ -1512,40 +1325,80 @@ on_auto_indent (GtkAction *action, gpointer data)
 		DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent);
 		set_line_indentation (editor, insert_line, line_indent);
 	}
-	ianjuta_editor_end_undo_action (editor, NULL);
+	ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 }
 
-static GtkActionEntry actions_indent[] = {
+static void
+on_auto_complete (GtkAction *action, gpointer data)
+{
+	CppJavaPlugin *lang_plugin;
+	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
+	if (lang_plugin->assist)
+		cpp_java_assist_check (lang_plugin->assist, TRUE, TRUE);
+}
+
+static GtkActionEntry actions[] = {
 	{
 		"ActionMenuEdit",
 		NULL, N_("_Edit"),
 		NULL, NULL, NULL
 	},
+	{
+		"ActionEditAutocomplete",
+		ANJUTA_STOCK_AUTOCOMPLETE,
+		N_("_AutoComplete"), "<control>Return",
+		N_("AutoComplete the current word"),
+		G_CALLBACK (on_auto_complete)
+	},
 	{
 		"ActionEditAutoindent",
-		NULL,
+		ANJUTA_STOCK_AUTOINDENT,
 		N_("Auto Indent"), "<control>i",
 		N_("Auto indent current line or selection based on indentation settings"),
 		G_CALLBACK (on_auto_indent)
 	}
 };
 
+static void
+register_stock_icons (AnjutaPlugin *plugin)
+{
+	static gboolean registered = FALSE;
+
+	if (registered)
+		return;
+	registered = TRUE;
+
+	/* Register stock icons */
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_COMPLETE, ANJUTA_STOCK_COMPLETE);	
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_AUTOCOMPLETE, ANJUTA_STOCK_AUTOCOMPLETE);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_AUTOINDENT, ANJUTA_STOCK_AUTOINDENT);
+	END_REGISTER_ICON (plugin);
+}
+
 static gboolean
 cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 {
 	AnjutaUI *ui;
 	CppJavaPlugin *lang_plugin;
+	static gboolean initialized = FALSE;
+	
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (plugin);
 	
 	DEBUG_PRINT ("AnjutaLanguageCppJavaPlugin: Activating plugin ...");
 
+	if (!initialized)
+	{
+		register_stock_icons (plugin);
+	}
+
 	lang_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	lang_plugin->action_group = 
-		anjuta_ui_add_action_group_entries (ui, "ActionGroupAutoindent",
-											_("Autoindent"),
-											actions_indent,
-											G_N_ELEMENTS (actions_indent),
+		anjuta_ui_add_action_group_entries (ui, "ActionGroupCppJavaAssist",
+											_("Cpp/Java Assistance"),
+											actions,
+											G_N_ELEMENTS (actions),
 											GETTEXT_PACKAGE, TRUE,
 											plugin);
 	lang_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
@@ -1556,10 +1409,7 @@ cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 								 on_value_added_current_editor,
 								 on_value_removed_current_editor,
 								 plugin);
-	
-	lang_plugin->symbol_browser = 
-		anjuta_shell_get_interface(plugin->shell, IAnjutaSymbolManager, NULL);
-	
+	initialized = FALSE;
 	return TRUE;
 }
 
@@ -1570,15 +1420,16 @@ cpp_java_plugin_deactivate_plugin (AnjutaPlugin *plugin)
 	CppJavaPlugin *lang_plugin;
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (plugin);
 	
+	anjuta_plugin_remove_watch (plugin,
+								lang_plugin->editor_watch_id,
+								TRUE);
+	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	anjuta_ui_unmerge (ui, lang_plugin->uiid);
 	anjuta_ui_remove_action_group (ui, lang_plugin->action_group);
 	
 	lang_plugin->action_group = NULL;
 	lang_plugin->uiid = 0;
-	anjuta_plugin_remove_watch (plugin,
-								lang_plugin->editor_watch_id,
-								TRUE);
 	DEBUG_PRINT ("AnjutaLanguageCppJavaPlugin: Deactivated plugin.");
 	return TRUE;
 }
@@ -1586,6 +1437,8 @@ cpp_java_plugin_deactivate_plugin (AnjutaPlugin *plugin)
 static void
 cpp_java_plugin_finalize (GObject *obj)
 {
+	/* CppJavaPlugin* plugin = ANJUTA_PLUGIN_CPP_JAVA (obj); */
+
 	/* Finalization codes here */
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
 }
@@ -1604,9 +1457,9 @@ cpp_java_plugin_instance_init (GObject *obj)
 	plugin->action_group = NULL;
 	plugin->current_editor = NULL;
 	plugin->current_language = NULL;
-	plugin->symbol_browser = NULL;
 	plugin->editor_watch_id = 0;
 	plugin->uiid = 0;
+	plugin->assist = NULL;
 }
 
 static void
@@ -1640,8 +1493,7 @@ static void
 ipreferences_unmerge (IAnjutaPreferences* ipref, AnjutaPreferences* prefs,
 					  GError** e)
 {
-	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs), 
-										  _("C/C++/Java"));
+	anjuta_preferences_remove_page(prefs, _("C/C++/Java"));
 }
 
 static void
