@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <libgnomevfs/gnome-vfs.h>
@@ -24,6 +24,8 @@
 #include "message-view.h"
 #define MESSAGE_TYPE message_get_type()
 
+#define HAVE_TOOLTIP_API (GTK_MAJOR_VERSION > 2 || (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 12))
+
 struct _MessageViewPrivate
 {
 	//guint num_messages;
@@ -41,10 +43,12 @@ struct _MessageViewPrivate
 	gchar *pixmap;
 	gboolean highlite;
 	
+#if !HAVE_TOOLTIP_API
 	GdkRectangle tooltip_rect;
 	GtkWidget *tooltip_window;
 	gulong tooltip_timeout;
 	PangoLayout *tooltip_layout;
+#endif
 	
 	/* gconf notification ids */
 	GList *gconf_notify_ids;
@@ -217,6 +221,49 @@ escape_string (const gchar *str)
 	return g_string_free (gstr, FALSE);
 }
 
+#if HAVE_TOOLTIP_API
+static gboolean
+message_view_query_tooltip (GtkWidget* widget, gint x, gint y, gboolean keyboard,
+						 GtkTooltip* tooltip)
+{
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	MessageView* view = MESSAGE_VIEW(widget);
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->privat->tree_view));
+	
+	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(view->privat->tree_view),
+		x, y, &path, NULL, NULL, NULL))
+	{
+		Message *message;
+		gchar *text, *title, *desc;
+		
+		gtk_tree_model_get_iter (model, &iter, path);
+		gtk_tree_model_get (model, &iter, COLUMN_MESSAGE, &message, -1); 
+		gtk_tree_path_free(path);
+		
+		if (!message->details || !message->summary ||
+			strlen (message->details) <= 0 ||
+			strlen (message->summary) <= 0)
+			return FALSE;
+		
+		title = escape_string (message->summary);
+		desc = escape_string (message->details);
+		text = g_strdup_printf ("<b>%s</b>\n%s", title, desc);
+		
+		g_free (title);
+		g_free (desc);
+		
+		gtk_tooltip_set_markup (tooltip, text);
+		g_free (text);
+		return TRUE;
+	}
+	return FALSE;
+}
+#endif
+
+#if !HAVE_TOOLTIP_API
 /* Tooltip operations -- taken from gtodo */
 
 static gchar *
@@ -430,6 +477,8 @@ tooltip_leave_cb (GtkWidget *w, GdkEventCrossing *e, MessageView *view)
 		view->privat->tooltip_window = NULL;
 	}
 }
+#endif
+
 
 /* MessageView signal callbacks */
 /* Send a signal if a message was double-clicked or ENTER or SPACE was pressed */
@@ -603,15 +652,18 @@ message_view_dispose (GObject *obj)
 		prefs_finalize (mview);
 		mview->privat->gconf_notify_ids = NULL;
 	}
+#if !HAVE_TOOLTIP_API
 	if (mview->privat->tooltip_timeout) {
 		g_source_remove (mview->privat->tooltip_timeout);
 		mview->privat->tooltip_timeout = 0;
 	}
+
 	if (mview->privat->tooltip_window) {
 		gtk_widget_destroy (mview->privat->tooltip_window);
 		g_object_unref (mview->privat->tooltip_layout);
 		mview->privat->tooltip_window = NULL;
 	}
+#endif
 	if (mview->privat->tree_view)
 	{
 		mview->privat->tree_view = NULL;
@@ -649,7 +701,7 @@ message_view_instance_init (MessageView * self)
 	self->privat->line_buffer = g_strdup("");
 
 	/* Create the tree widget */
-	model = gtk_list_store_new (N_COLUMNS, GDK_TYPE_COLOR,
+	model = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING,
 								G_TYPE_STRING, MESSAGE_TYPE,  G_TYPE_STRING);
 	self->privat->tree_view =
 		gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
@@ -675,7 +727,7 @@ message_view_instance_init (MessageView * self)
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_title (column, _("Messages"));
 	gtk_tree_view_column_add_attribute
-		(column, renderer, "foreground-gdk", COLUMN_COLOR);
+		(column, renderer, "foreground", COLUMN_COLOR);
 	gtk_tree_view_column_add_attribute
 		(column, renderer, "markup", COLUMN_SUMMARY);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (self->privat->tree_view),
@@ -707,10 +759,14 @@ message_view_instance_init (MessageView * self)
 	/* Connect signals */
 	g_signal_connect (G_OBJECT(self->privat->tree_view), "event", 
 					  G_CALLBACK (on_message_event), self);
+#if !HAVE_TOOLTIP_API
 	g_signal_connect (G_OBJECT (self->privat->tree_view), "motion-notify-event",
 					  G_CALLBACK (tooltip_motion_cb), self);
 	g_signal_connect (G_OBJECT (self->privat->tree_view), "leave-notify-event",
 					  G_CALLBACK (tooltip_leave_cb), self);
+#else
+	g_object_set (G_OBJECT(self), "has-tooltip", TRUE, NULL);
+#endif
 	g_object_unref (model);
 }
 
@@ -721,6 +777,7 @@ message_view_class_init (MessageViewClass * klass)
 	GParamSpec *message_view_spec_pixmap;
 	GParamSpec *message_view_spec_highlite;
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass* widget_class = GTK_WIDGET_CLASS (klass);
 	
 	parent_class = g_type_class_peek_parent (klass);
 	gobject_class->set_property = message_view_set_property;
@@ -728,6 +785,10 @@ message_view_class_init (MessageViewClass * klass)
 	gobject_class->finalize = message_view_finalize;
 	gobject_class->dispose = message_view_dispose;
 	
+#if HAVE_TOOLTIP_API
+	widget_class->query_tooltip = message_view_query_tooltip;
+#endif
+	
 	message_view_spec_label = g_param_spec_string ("label",
 						       "Label of the view",
 						       "Used to decorate the view,"
@@ -1021,12 +1082,12 @@ static void
 pref_change_color (MessageView *mview, IAnjutaMessageViewType type,
 				   const gchar *color_pref_key)
 {
-	GdkColor *color;
+	gchar* color;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gboolean success;
 	
-	color = anjuta_util_convert_color (mview->privat->prefs, color_pref_key);
+	color = anjuta_preferences_get (mview->privat->prefs, color_pref_key);
 	store = GTK_LIST_STORE (gtk_tree_view_get_model
 				(GTK_TREE_VIEW (mview->privat->tree_view)));
 	success = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
@@ -1044,14 +1105,6 @@ pref_change_color (MessageView *mview, IAnjutaMessageViewType type,
 	g_free(color);
 }
 
-static void
-on_gconf_notify_color_info (GConfClient *gclient, guint cnxn_id,
-							GConfEntry *entry, gpointer user_data)
-{
-	pref_change_color (MESSAGE_VIEW (user_data),
-					   IANJUTA_MESSAGE_VIEW_TYPE_INFO,
-					   "messages.color.info");
-}
 
 static void
 on_gconf_notify_color_warning (GConfClient *gclient, guint cnxn_id,
@@ -1080,7 +1133,6 @@ static void
 prefs_init (MessageView *mview)
 {
 	guint notify_id;
-	REGISTER_NOTIFY ("messages.color.info", on_gconf_notify_color_info);
 	REGISTER_NOTIFY ("messages.color.warning", on_gconf_notify_color_warning);
 	REGISTER_NOTIFY ("messages.color.error", on_gconf_notify_color_error);
 }
@@ -1152,7 +1204,7 @@ imessage_view_append (IAnjutaMessageView *message_view,
 					  const gchar *details,
 					  GError ** e)
 {
-	GdkColor* color;
+	gchar* color;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gboolean highlite;
@@ -1176,18 +1228,16 @@ imessage_view_append (IAnjutaMessageView *message_view,
 		switch (message->type)
 		{
 			case IANJUTA_MESSAGE_VIEW_TYPE_INFO:
-				color = anjuta_util_convert_color(view->privat->prefs,
-									  "messages.color.info");
 				stock_id = GTK_STOCK_INFO;
 				break;
 			case IANJUTA_MESSAGE_VIEW_TYPE_WARNING:
-				color = anjuta_util_convert_color(view->privat->prefs,
+				color = anjuta_preferences_get (view->privat->prefs,
 									  "messages.color.warning");
 				/* FIXME: There is no GTK_STOCK_WARNING which would fit better here */
 				stock_id = GTK_STOCK_DIALOG_WARNING;
 				break;
 			case IANJUTA_MESSAGE_VIEW_TYPE_ERROR:
-				color = anjuta_util_convert_color(view->privat->prefs,
+				color = anjuta_preferences_get (view->privat->prefs,
 									  "messages.color.error");
 				stock_id = GTK_STOCK_STOP;
 				break;
@@ -1216,23 +1266,21 @@ imessage_view_append (IAnjutaMessageView *message_view,
 	} else {
 		escaped_str = escape_string (message->summary);
 	}
+	gtk_list_store_set (store, &iter,
+							COLUMN_SUMMARY, escaped_str,
+							COLUMN_MESSAGE, message,
+							-1);
 	if (color)
 	{
 		gtk_list_store_set (store, &iter,
-							COLUMN_COLOR, color,
-							COLUMN_SUMMARY, escaped_str,
-							COLUMN_MESSAGE, message,
-							COLUMN_PIXBUF, stock_id,
-							-1);
-	
-		/* Can we free the color when it's in the tree_view? */
-		// gdk_color_free (color);
-	} else {
-		gtk_list_store_set (store, &iter,
-							COLUMN_SUMMARY, escaped_str,
-							COLUMN_MESSAGE, message,
-							-1);
+							COLUMN_COLOR, color, -1);
 	}
+	if (stock_id)
+	{
+		gtk_list_store_set (store, &iter,
+							COLUMN_PIXBUF, stock_id, -1);
+	}
+	g_free(color);
 	message_free (message);
 	g_free (utf8_msg);
 	g_free (escaped_str);
