@@ -17,11 +17,12 @@
  * You should have received a copy of the GNU General Public License
  * along with plugin.c.  See the file "COPYING".  If not,
  * write to:  The Free Software Foundation, Inc.,
- *            59 Temple Place - Suite 330,
- *            Boston,  MA  02111-1307, USA.
+ *            51 Franklin Street, Fifth Floor,
+ *            Boston,  MA  02110-1301, USA.
  */
 
 #include "gprof-call-graph-view.h"
+#include <glib/gi18n-lib.h>
 
 struct _GProfCallGraphViewPriv
 {
