@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -992,19 +992,9 @@ on_treeview_event  (GtkWidget *widget,
 	return FALSE;
 }
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -1012,10 +1002,10 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/"ICON_FILE,
+	BEGIN_REGISTER_ICON(plugin);
+	REGISTER_ICON (ICON_FILE,
 				   "project-manager-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static void
@@ -1069,6 +1059,9 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 	ProjectManagerPlugin *pm_plugin;
 	
 	editor = g_value_get_object (value);
+	if (!IANJUTA_IS_EDITOR(editor))
+		return;
+	
 	pm_plugin = ANJUTA_PLUGIN_PROJECT_MANAGER (plugin);
 	
 	if (pm_plugin->current_editor_uri)
@@ -1219,12 +1212,17 @@ project_manager_unload_gbf (ProjectManagerPlugin *pm_plugin)
 			GList *node;
 			
 			editors =
-				ianjuta_document_manager_get_editors (IANJUTA_DOCUMENT_MANAGER
+				ianjuta_document_manager_get_documents (IANJUTA_DOCUMENT_MANAGER
 													  (docman), NULL);
 			node = editors;
 			while (node)
 			{
-				const gchar *editor_uri =
+				if (!IANJUTA_IS_EDITOR(node->data))
+				{
+					node = g_list_next(node);
+					continue;
+				}
+				gchar *editor_uri =
 					ianjuta_file_get_uri (IANJUTA_FILE (node->data), NULL);
 				
 				/* Only remove if it does not have unsaved data */
@@ -1240,13 +1238,14 @@ project_manager_unload_gbf (ProjectManagerPlugin *pm_plugin)
 						to_remove = g_list_prepend (to_remove, node->data);
 					}
 				}
+				g_free (editor_uri);
 				node = g_list_next (node);
 			}
 			node = to_remove;
 			while (node)
 			{
-				ianjuta_document_manager_remove_buffer (docman,
-												IANJUTA_EDITOR (node->data),
+				ianjuta_document_manager_remove_document (docman,
+												IANJUTA_DOCUMENT (node->data),
 														FALSE,
 														NULL);
 				node = g_list_next (node);
@@ -1288,6 +1287,7 @@ on_profile_scoped (AnjutaProfileManager *profile_manager,
 	g_value_init (value, G_TYPE_STRING);
 	g_value_set_string (value, plugin->project_root_uri);
 	
+	update_title (plugin, plugin->project_root_uri);
 	anjuta_shell_add_value (ANJUTA_PLUGIN(plugin)->shell,
 							"project_root_uri",
 							value, NULL);
@@ -1326,8 +1326,6 @@ on_profile_descoped (AnjutaProfileManager *profile_manager,
 	project_manager_save_session (plugin);
 	
 	/* Close current project */
-	anjuta_shell_remove_value (ANJUTA_PLUGIN (plugin)->shell,
-							   "project_root_uri", NULL);
 	project_manager_unload_gbf (plugin);
 		
 	g_free (plugin->project_root_uri);
@@ -1336,6 +1334,8 @@ on_profile_descoped (AnjutaProfileManager *profile_manager,
 	plugin->project_root_uri = NULL;
 	
 	update_title (ANJUTA_PLUGIN_PROJECT_MANAGER (plugin), NULL);
+	anjuta_shell_remove_value (ANJUTA_PLUGIN (plugin)->shell,
+							   "project_root_uri", NULL);
 }
 
 static void
@@ -2353,7 +2353,6 @@ ifile_open (IAnjutaFile *ifile, const gchar* uri, GError **e)
 	anjuta_shell_thaw (ANJUTA_PLUGIN (ifile)->shell, NULL);
 #endif
 	anjuta_status_progress_tick (status, NULL, _("Loaded Project..."));
-	update_title (plugin, plugin->project_root_uri);
 }
 
 static gchar*
