@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA 02110-1301, USA.
  */
  
 /*
@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-document.c 2964 2007-05-14 19:58:50Z jhs $
+ * $Id: anjuta-document.c 3181 2007-09-27 14:09:40Z jhs $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +42,6 @@
 #include <libanjuta/anjuta-debug.h>
 
 #include "anjuta-encodings.h"
-#include "anjuta-languages-manager.h"
 #include "anjuta-document.h"
 #include "anjuta-document-loader.h"
 #include "anjuta-document-saver.h"
@@ -50,6 +49,7 @@
 #include "anjuta-utils.h"
 
 #include <gtksourceview/gtksourceiter.h>
+#include <gtksourceview/gtksourcelanguagesmanager.h>
 #include <pcre.h>
 
 #define ANJUTA_MAX_PATH_LEN  2048
@@ -77,8 +77,8 @@ struct _AnjutaDocumentPrivate
 	GnomeVFSURI *vfs_uri;
 
 	const AnjutaEncoding *encoding;
+	GtkSourceLanguagesManager* lang_manager;
 
-	gchar	    *mime_type;
 
 	time_t       mtime;
 
@@ -100,7 +100,6 @@ enum {
 
 	PROP_URI,
 	PROP_SHORTNAME,
-	PROP_MIME_TYPE,
 	PROP_READ_ONLY,
 	PROP_ENCODING
 };
@@ -180,31 +179,17 @@ anjuta_document_finalize (GObject *object)
 	{
 		GtkTextIter iter;
 		// gchar *position;
-		gchar *lang_id = NULL;
-		GtkSourceLanguage *lang;
 
 		gtk_text_buffer_get_iter_at_mark (
 				GTK_TEXT_BUFFER (doc),			
 				&iter,
 				gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
-
-		if (doc->priv->language_set_by_user)
-		{
-			lang = anjuta_document_get_language (doc);
-
-			if (lang != NULL)
-				lang_id = gtk_source_language_get_id (lang);
-
-			g_free (lang_id);
-		}
 	}
 
 	g_free (doc->priv->uri);
 	if (doc->priv->vfs_uri != NULL)
 		gnome_vfs_uri_unref (doc->priv->vfs_uri);
 
-	g_free (doc->priv->mime_type);
-
 	if (doc->priv->loader)
 		g_object_unref (doc->priv->loader);
 }
@@ -225,9 +210,6 @@ anjuta_document_get_property (GObject    *object,
 		case PROP_SHORTNAME:
 			g_value_take_string (value, anjuta_document_get_short_name_for_display (doc));
 			break;
-		case PROP_MIME_TYPE:
-			g_value_set_string (value, doc->priv->mime_type);
-			break;
 		case PROP_READ_ONLY:
 			g_value_set_boolean (value, doc->priv->readonly);
 			break;
@@ -318,12 +300,6 @@ anjuta_document_class_init (AnjutaDocumentClass *klass)
 					 		      "The document's short name",
 					 		      NULL,
 					 		      G_PARAM_READABLE));
-	g_object_class_install_property (object_class, PROP_MIME_TYPE,
-					 g_param_spec_string ("mime-type",
-					 		      "MIME Type",
-					 		      "The document's MIME Type",
-					 		      "text/plain",
-					 		      G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_READ_ONLY,
 					 g_param_spec_boolean ("read-only",
 					 		       "Read Only",
@@ -402,42 +378,6 @@ anjuta_document_class_init (AnjutaDocumentClass *klass)
 	g_type_class_add_private (object_class, sizeof(AnjutaDocumentPrivate));
 }
 
-static void
-set_language (AnjutaDocument     *doc, 
-              GtkSourceLanguage *lang,
-              gboolean           set_by_user)
-{
-	GtkSourceLanguage *old_lang;
-	
-	old_lang = gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (doc));
-	
-	if (old_lang == lang)
-		return;
-
-	gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (doc), lang);
-		
-	if (lang != NULL)
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc),
-						 TRUE);
-	else
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc), 
-						 FALSE);
-
-	if (set_by_user && (doc->priv->uri != NULL))
-	{
-		gchar *lang_id = NULL;
-
-		if (lang != NULL)
-		{
-			lang_id = gtk_source_language_get_id (lang);
-			g_return_if_fail (lang_id != NULL);
-		}
-
-		g_free (lang_id);
-	}
-	
-	doc->priv->language_set_by_user = set_by_user;
-}
 
 static void
 set_encoding (AnjutaDocument       *doc, 
@@ -471,8 +411,6 @@ anjuta_document_init (AnjutaDocument *doc)
 	doc->priv->vfs_uri = NULL;
 	doc->priv->untitled_number = get_untitled_number ();
 
-	doc->priv->mime_type = g_strdup ("text/plain");
-
 	doc->priv->readonly = FALSE;
 
 	doc->priv->stop_cursor_moved_emission = FALSE;
@@ -502,12 +440,9 @@ anjuta_document_new (void)
 	return ANJUTA_DOCUMENT (g_object_new (ANJUTA_TYPE_DOCUMENT, NULL));
 }
 
-/* If mime type is null, we guess from the filename */
-/* If uri is null, we only set the mime-type */
 static void
 set_uri (AnjutaDocument *doc,
-	 const gchar   *uri,
-	 const gchar   *mime_type)
+	 const gchar   *uri)
 {
 
 	g_return_if_fail ((uri == NULL) || anjuta_utils_is_valid_uri (uri));
@@ -533,54 +468,6 @@ set_uri (AnjutaDocument *doc,
 			doc->priv->untitled_number = 0;
 		}
 	}
-
-	g_free (doc->priv->mime_type);
-	if (mime_type != NULL)
-	{
-		doc->priv->mime_type = g_strdup (mime_type);
-	}
-	else
-	{
-		gchar *base_name = NULL;
-
-		/* Guess the mime type from file extension or fallback to "text/plain" */
-		if (doc->priv->vfs_uri != NULL)
-			base_name = gnome_vfs_uri_extract_short_path_name (doc->priv->vfs_uri);
-		if (base_name != NULL)
-		{
-			const gchar *detected_mime;
-
-			detected_mime = gnome_vfs_get_mime_type_for_name (base_name);
-			if (detected_mime == NULL ||
-			    strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, detected_mime) == 0)
-				detected_mime = "text/plain";
-
-			doc->priv->mime_type = g_strdup (detected_mime);
-
-			g_free (base_name);
-		}
-		else
-		{
-			doc->priv->mime_type = g_strdup ("text/plain");
-		}
-	}
-
-	if (!doc->priv->language_set_by_user)
-	{
-		// gchar *data;
-		GtkSourceLanguage *language = NULL;
-
-		
-		if (strcmp (doc->priv->mime_type, "text/plain") != 0)
-		{
-			language = gtk_source_languages_manager_get_language_from_mime_type (
-						anjuta_get_languages_manager (),
-						doc->priv->mime_type);
-		}
-
-		set_language (doc, language, FALSE);
-	}
-
 	g_object_notify (G_OBJECT (doc), "uri");
 	g_object_notify (G_OBJECT (doc), "shortname");
 }
@@ -689,16 +576,6 @@ anjuta_document_get_short_name_for_display (AnjutaDocument *doc)
 		return get_uri_shortname_for_display (doc->priv->vfs_uri);
 }
 
-/* Never returns NULL */
-gchar *
-anjuta_document_get_mime_type (AnjutaDocument *doc)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), "text/plain");
-	g_return_val_if_fail (doc->priv->mime_type != NULL, "text/plain");
-
- 	return g_strdup (doc->priv->mime_type);
-}
-
 /* Note: do not emit the notify::read-only signal */
 static void
 set_readonly (AnjutaDocument *doc,
@@ -751,9 +628,6 @@ document_loader_loaded (AnjutaDocumentLoader *loader,
 	if (error == NULL)
 	{
 		GtkTextIter iter;
-		const gchar *mime_type;
-
-		mime_type = anjuta_document_loader_get_mime_type (loader);
 
 		doc->priv->mtime = anjuta_document_loader_get_mtime (loader);
 
@@ -767,7 +641,7 @@ document_loader_loaded (AnjutaDocumentLoader *loader,
 			      (doc->priv->requested_encoding != NULL));
 		      
 		/* We already set the uri */
-		set_uri (doc, NULL, mime_type);
+		set_uri (doc, NULL);
 
 		/* move the cursor at the requested line if any */
 		if (doc->priv->requested_line_pos > 0)
@@ -857,7 +731,7 @@ anjuta_document_load (AnjutaDocument       *doc,
 	doc->priv->requested_encoding = encoding;
 	doc->priv->requested_line_pos = line_pos;
 
-	set_uri (doc, uri, NULL);
+	set_uri (doc, uri);
 
 	anjuta_document_loader_load (doc->priv->loader,
 				    uri,
@@ -888,10 +762,8 @@ document_saver_saving (AnjutaDocumentSaver *saver,
 		if (error == NULL)
 		{
 			const gchar *uri;
-			const gchar *mime_type;
 
 			uri = anjuta_document_saver_get_uri (saver);
-			mime_type = anjuta_document_saver_get_mime_type (saver);
 
 			doc->priv->mtime = anjuta_document_saver_get_mtime (saver);
 
@@ -902,7 +774,7 @@ document_saver_saving (AnjutaDocumentSaver *saver,
 			gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc),
 						      FALSE);
 
-			set_uri (doc, uri, mime_type);
+			set_uri (doc, uri);
 
 			set_encoding (doc, 
 				      doc->priv->requested_encoding, 
@@ -990,42 +862,6 @@ anjuta_document_save_as (AnjutaDocument          *doc,
 	document_save_real (doc, uri, encoding, 0, flags);
 }
 
-gboolean
-anjuta_document_insert_file (AnjutaDocument       *doc,
-			    GtkTextIter         *iter,
-			    const gchar         *uri,
-			    const AnjutaEncoding *encoding)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), FALSE);
-	g_return_val_if_fail (iter != NULL, FALSE);
-	g_return_val_if_fail (gtk_text_iter_get_buffer (iter) == 
-				GTK_TEXT_BUFFER (doc), FALSE);
-
-	// TODO
-
-	return FALSE;
-}
-
-gboolean	 
-anjuta_document_is_untouched (AnjutaDocument *doc)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), TRUE);
-
-	return (doc->priv->uri == NULL) && 
-		(!gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)));
-}
-
-gboolean
-anjuta_document_get_deleted (AnjutaDocument *doc)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), FALSE);
-
-	if (doc->priv->uri == NULL || doc->priv->vfs_uri == NULL)
-		return FALSE;
-		
-	return !gnome_vfs_uri_exists (doc->priv->vfs_uri);
-}
-
 /*
  * If @line is bigger than the lines of the document, the cursor is moved
  * to the last line and FALSE is returned.
@@ -1061,23 +897,6 @@ anjuta_document_goto_line (AnjutaDocument *doc,
 	return ret;
 }
 
-void
-anjuta_document_set_language (AnjutaDocument     *doc, 
-			     GtkSourceLanguage *lang)
-{
-	g_return_if_fail (ANJUTA_IS_DOCUMENT (doc));
-
-	set_language (doc, lang, TRUE);
-}
-
-GtkSourceLanguage *
-anjuta_document_get_language (AnjutaDocument *doc)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), NULL);
-
-	return gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (doc));
-}
-
 const AnjutaEncoding *
 anjuta_document_get_encoding (AnjutaDocument *doc)
 {
@@ -1086,84 +905,63 @@ anjuta_document_get_encoding (AnjutaDocument *doc)
 	return doc->priv->encoding;
 }
 
-#define WORD_REGEX "[^ \\t&*!(]+$"
-gchar* anjuta_document_get_current_word(AnjutaDocument* doc)
-{
-	pcre *re;
-  gint err_offset;
-	const gchar* err;
-	gint rc, start, end;
-	int ovector[2];
-	gchar* line, *word;
-	GtkTextIter *line_iter, cursor_iter;
-	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(doc);
-	gtk_text_buffer_get_iter_at_mark(buffer, &cursor_iter, 
-								 gtk_text_buffer_get_insert(buffer));
-	
-	line_iter = gtk_text_iter_copy(&cursor_iter);
-	gtk_text_iter_set_line_offset(line_iter, 0);
-	line = gtk_text_buffer_get_text(buffer, line_iter, &cursor_iter, FALSE);
-	gtk_text_iter_free(line_iter);
 
-	/* Create regular expression */
-	re = pcre_compile(WORD_REGEX, 0, &err, &err_offset, NULL);
+static gboolean
+wordcharacters_contains (gchar c)
+{
+	const gchar* wordcharacters =
+		"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	gint pos;
 	
-	if (NULL == re)
+	for (pos = 0; pos < strlen(wordcharacters); pos++)
 	{
-		/* Compile failed - check error message */
-		DEBUG_PRINT("Regex compile failed! %s at position %d", err, err_offset);
-		return FALSE;
+		if (wordcharacters[pos] == c)
+		{
+			return TRUE;
+		}
 	}
-	
-	/* Run the next matching operation */
-	rc = pcre_exec(re, NULL, line, strlen(line), 0,
-					   0, ovector, 2);
-
-	 if (rc == PCRE_ERROR_NOMATCH)
-    {
-    	DEBUG_PRINT("No match");
-    	g_free(line);
-    	return NULL;
-    }
- 	 else if (rc < 0)
-    {
-    	DEBUG_PRINT("Matching error %d\n", rc);
-    	return NULL;
-    }
-	else if (rc == 0)
-    {
-    	DEBUG_PRINT("ovector too small");
-    	return NULL;
-    }
-    start = ovector[0];
-    end = ovector[1];
-    	    	
-    word = g_new0(gchar, end - start + 1);
-    strncpy(word, line + start, end - start);
-  	
-  	g_free(line);
-  	
-  	return word;
+	return FALSE;
 }
 
-glong
-_anjuta_document_get_seconds_since_last_save_or_load (AnjutaDocument *doc)
+gchar* anjuta_document_get_current_word(AnjutaDocument* doc, gboolean end_position)
 {
-	GTimeVal current_time;
+	GtkTextIter begin;
+	GtkTextIter end;
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(doc);
+	gchar* region;
+	gchar* word;
+	gint startword;
+	gint endword;
+	int cplen;
+	const int maxlength = 100;
 	
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), -1);
-
-	g_get_current_time (&current_time);
-
-	return (current_time.tv_sec - doc->priv->time_of_last_save_or_load.tv_sec);
-}
-
-gboolean
-_anjuta_document_is_saving_as (AnjutaDocument *doc)
-{	
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), FALSE);
+	gtk_text_buffer_get_iter_at_mark (buffer, &begin, 
+																		gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)));
+	gtk_text_buffer_get_iter_at_mark (buffer, &end, 
+																		gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)));
+	startword = gtk_text_iter_get_line_offset (&begin);	
+	endword = gtk_text_iter_get_line_offset (&end);
 	
-	return (doc->priv->is_saving_as);
+	gtk_text_iter_set_line_offset (&begin, 0);
+	gtk_text_iter_forward_to_line_end (&end);
+	
+	region = gtk_text_buffer_get_text (buffer, &begin, &end, FALSE);
+	
+	while (startword> 0 && wordcharacters_contains(region[startword - 1]))
+		startword--;
+	while (!end_position && region[endword] && wordcharacters_contains(region[endword]))
+		endword++;
+	if(startword == endword)
+		return NULL;
+	
+	region[endword] = '\0';
+	cplen = (maxlength < (endword-startword+1))?maxlength:(endword-startword+1);
+	word = g_strndup (region + startword, cplen);
+	
+	DEBUG_PRINT ("region: %s\n start: %d end: %d word: %s", region, startword, endword, word);
+	g_free(region);
+	
+	return word;
 }
 						 
 static void	
