@@ -11,7 +11,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
  
 #ifndef SOURCEVIEW_PRIVATE_H
@@ -21,6 +21,7 @@
 #include "anjuta-document.h"
 
 #include "assist-window.h"
+#include "assist-tip.h"
 
 #include <libanjuta/anjuta-plugin.h>
 #include <glib.h>
@@ -62,8 +63,8 @@ struct SourceviewPrivate {
 	gint goto_line;
 	
 	/* Assist */
-	GHashTable* triggers;
 	AssistWindow* assist_win;
+	AssistTip* assist_tip;
 	
 	/* Plugin */
 	AnjutaPlugin* plugin;
