@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -68,19 +68,9 @@ static void on_editor_update_ui (IAnjutaEditor *editor,
 static void on_char_added (IAnjutaEditor *editor, gint position, gchar ch,
 						   SymbolBrowserPlugin *sv_plugin);
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -88,9 +78,9 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/"ICON_FILE, "symbol-browser-plugin-icon");
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON (ICON_FILE, "symbol-browser-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static void
@@ -754,6 +744,7 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 		anjuta_symbol_view_workspace_add_file (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree), uri);
 		action = anjuta_ui_get_action (ui, "ActionGroupSymbolNavigation",
 									   "ActionGotoSymbol");
+		g_free (uri);
 		
 		file_symbol_model =
 			anjuta_symbol_view_get_file_symbol_model (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree));
@@ -777,8 +768,6 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 				g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 		}
 	}
-	if (uri)
-		g_free (uri);
 }
 
 static gboolean
@@ -906,6 +895,9 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 	
 	editor = g_value_get_object (value);
 	
+	if (!IANJUTA_IS_EDITOR(editor))
+		return;
+	
 	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	
 	if (!sv_plugin->editor_connected)
@@ -926,8 +918,8 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 						   sv_plugin);
 		if (uri)
 		{
-			g_hash_table_insert (sv_plugin->editor_connected, editor,
-								 g_strdup (uri));
+			g_hash_table_insert (sv_plugin->editor_connected, editor, uri);	//g_strdup (uri));
+			//g_free (uri);
 		}
 		else
 		{
@@ -945,7 +937,6 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 						  G_CALLBACK (on_editor_update_ui),
 						  sv_plugin);
 	}
-	g_free (uri);
 	
 	/* add a default timeout to the updating of buffer symbols */	
 	timeout_id = g_timeout_add (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
@@ -1023,6 +1014,8 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	sv_plugin->sv_tab_label = gtk_label_new (_("Global" ));
 	sv_plugin->sv_tree = anjuta_symbol_view_new ();
+	symbol_browser_load_global_tags (plugin);
+	
 	g_object_add_weak_pointer (G_OBJECT (sv_plugin->sv_tree),
 							   (gpointer)&sv_plugin->sv_tree);
 
@@ -1111,7 +1104,6 @@ activate_plugin (AnjutaPlugin *plugin)
 		anjuta_plugin_add_watch (plugin, "document_manager_current_editor",
 								 value_added_current_editor,
 								 value_removed_current_editor, NULL);
-	
 	return TRUE;
 }
 
@@ -1382,7 +1374,6 @@ isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
 	return NULL;
 }
 
-
 static void
 isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 {
