@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -66,7 +66,7 @@
 
 extern char **environ;
 
-extern GType terminal_plugin_get_type (AnjutaGluePlugin *plugin);
+extern GType terminal_plugin_get_type (GTypeModule *module);
 #define ANJUTA_PLUGIN_TERMINAL_TYPE         (terminal_plugin_get_type (NULL))
 #define ANJUTA_PLUGIN_TERMINAL(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_PLUGIN_TERMINAL_TYPE, TerminalPlugin))
 #define ANJUTA_PLUGIN_TERMINAL_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), ANJUTA_PLUGIN_TERMINAL_CLASS, TerminalPluginClass))
@@ -151,18 +151,25 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 											  GCONF_VALUE_STRING, NULL);
 	if (profiles)
 	{
-		GList *list = NULL;
-		GSList *node = profiles;
-		while (node)
+		if (term->pref_profile_combo)
 		{
-			if (node->data)
-				list = g_list_append (list, node->data);
-			node = g_slist_next (node);
+			GList *list = NULL;
+			GSList *node = profiles;
+			while (node)
+			{
+				if (node->data)
+					list = g_list_append (list, node->data);
+				node = g_slist_next (node);
+			}
+			gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo),
+										   list);
+			g_list_free (list);
 		}
-		gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo), list);
-		g_list_free (list);
+		g_slist_foreach (profiles, (GFunc)g_free, NULL);
+		g_slist_free (profiles);
 	}
-	setting = anjuta_preferences_get_int (pref, PREFS_TERMINAL_PROFILE_USE_DEFAULT);
+	setting = anjuta_preferences_get_int (pref,
+										  PREFS_TERMINAL_PROFILE_USE_DEFAULT);
 	if (setting)
 	{
 		/* Use the currently selected profile in gnome-terminal */
@@ -562,29 +569,18 @@ terminal_create (TerminalPlugin *term_plugin)
 	term_plugin->hbox = hbox;
 }
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
 		return;
 	registered = TRUE;
 
-	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON (PACKAGE_PIXMAPS_DIR"/"ICON_FILE, "terminal-plugin-icon");
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON (ICON_FILE, "terminal-plugin-icon");
+	END_REGISTER_ICON;
 }
 
 static gboolean
@@ -672,6 +668,7 @@ terminal_plugin_instance_init (GObject *obj)
 	TerminalPlugin *plugin = ANJUTA_PLUGIN_TERMINAL (obj);
 	plugin->gconf_notify_ids = NULL;
 	plugin->child_initizlized = FALSE;
+	plugin->pref_profile_combo = NULL;
 #if OLD_VTE == 1
 	plugin->first_time_realization = TRUE;
 #endif
@@ -737,8 +734,8 @@ ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError
 	TerminalPlugin* term_plugin = ANJUTA_PLUGIN_TERMINAL (ipref);
 	g_signal_handlers_disconnect_by_func(G_OBJECT(term_plugin->pref_default_button),
 		G_CALLBACK (use_default_profile_cb), term_plugin);
-	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs),
-		_("Terminal"));
+	anjuta_preferences_remove_page(prefs, _("Terminal"));
+	term_plugin->pref_profile_combo = NULL;
 }
 
 static void
