@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /*
@@ -713,7 +713,7 @@ on_editor_response (GtkDialog *dialog, gint response, gpointer user_data)
 			anjuta_util_dialog_error(GTK_WINDOW (this->dialog), _("A tool with the same name already exists!"));
 			return;
 		}
-
+		g_free (name);
 		
 		if (this->shortcut == NULL)
 		{
@@ -779,7 +779,7 @@ on_editor_response (GtkDialog *dialog, gint response, gpointer user_data)
 		if (gtk_toggle_button_get_active (this->script_tb))
 		{
 			IAnjutaDocumentManager *docman;
-			IAnjutaEditor *editor;
+			IAnjutaDocument *editor;
 
 			/* Check that default script directory exist */
 			data = g_build_filename (g_get_home_dir(), LOCAL_ANJUTA_SCRIPT_DIRECTORY, NULL);
@@ -816,7 +816,9 @@ on_editor_response (GtkDialog *dialog, gint response, gpointer user_data)
 				return;
 			}
 
-			editor = ianjuta_document_manager_find_editor_with_path (docman, data, NULL);
+			editor = 
+				ianjuta_document_manager_find_document_with_path 
+							   (docman, data, NULL);
 			if (editor == NULL)
 			{
 				IAnjutaFileLoader* loader;
@@ -831,15 +833,15 @@ on_editor_response (GtkDialog *dialog, gint response, gpointer user_data)
 			else
 			{
 				/* Set as current */
-				ianjuta_document_manager_set_current_editor (docman, editor, NULL);
+				ianjuta_document_manager_set_current_document 
+					(docman, IANJUTA_DOCUMENT(editor), NULL);
 			}
 			g_free (data);
 		}
-
-		atp_tool_dialog_refresh (this->parent, name);
-		g_free(name);
 	}
-
+	
+	atp_tool_dialog_refresh (this->parent, atp_user_tool_get_name (this->tool));
+	
 	atp_tool_editor_free (this);
 }
 
