@@ -17,8 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with plugin.c.  See the file "COPYING".  If not,
  * write to:  The Free Software Foundation, Inc.,
- *            59 Temple Place - Suite 330,
- *            Boston,  MA  02111-1307, USA.
+ *            51 Franklin Street, Fifth Floor,
+ *            Boston,  MA  02110-1301, USA.
  */
 
 #include <config.h>
@@ -42,12 +42,6 @@
 #define GLADE_FILE PACKAGE_DATA_DIR"/glade/anjuta-valgrind.glade"
 #define ICON_FILE PACKAGE_PIXMAPS_DIR"/anjuta-valgrind.png"
 
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
-
 
 enum {
 	MEMCHECK_OPTION,
@@ -61,10 +55,6 @@ static gpointer parent_class;
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -72,9 +62,9 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
+	BEGIN_REGISTER_ICON (plugin);
 	REGISTER_ICON ("anjuta-valgrind-knight.png", "valgrind-knight");
+	END_REGISTER_ICON;
 }
 
 
@@ -667,7 +657,7 @@ ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError**
 	pixbuf = gdk_pixbuf_new_from_file (ICON_FILE, NULL);
 
 	g_object_ref(valgrind->general_prefs);
-	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (prefs),
+	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (anjuta_preferences_get_dialog (prefs)),
 						"Valgrind", _("Valgrind"), pixbuf, valgrind->general_prefs);
 	g_object_unref (pixbuf);
 }
@@ -677,7 +667,7 @@ ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError
 {
 	AnjutaValgrindPlugin* valgrind = ANJUTA_PLUGIN_VALGRIND (ipref);
 
-	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs), 
+	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG (anjuta_preferences_get_dialog (prefs)), 
 		_("Valgrind"));
 
 	g_object_unref (valgrind->general_prefs);
