@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifdef HAVE_CONFIG_H
@@ -30,7 +30,7 @@
 
 #include <gnome.h>
 #include <gtk/gtkwindow.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-file-loader.h>
@@ -338,7 +338,7 @@ main (int argc, char *argv[])
 								  GNOME_PARAM_HUMAN_READABLE_NAME,
 								  _("Integrated Development Environment"),
 								  GNOME_PARAM_APP_DATADIR, data_dir,
-								  NULL);
+								  GNOME_PARAM_NONE);
 	g_free (data_dir);
 	
 	/* Get the command line files */
